import gflags
import logging

from ct.client import entry_decoder
from ct.client import state
from ct.client import aggregated_reporter
from ct.crypto import error
from ct.crypto import merkle
from ct.client import db_reporter
from ct.client import text_reporter
from ct.proto import client_pb2
from twisted.internet import defer
from twisted.internet import threads

FLAGS = gflags.FLAGS

gflags.DEFINE_integer("entry_write_batch_size", 1000, "Maximum number of "
                      "entries to batch into one database write")

class Monitor(object):
    def __init__(self, client, verifier, hasher, db, cert_db, log_key,
                 state_keeper):
        self.__client = client
        self.__verifier = verifier
        self.__hasher = hasher
        self.__db = db
        self.__state_keeper = state_keeper

        # TODO(ekasper): once consistency checks are in place, also load/store
        # Merkle tree info.
        # Depends on: Merkle trees implemented in Python.
        self.__state = client_pb2.MonitorState()
        self.__report = aggregated_reporter.AggregatedCertificateReport(
                (text_reporter.TextCertificateReport(),
                 db_reporter.CertDBCertificateReport(cert_db, log_key)))
        try:
            self.__state = self.__state_keeper.read(client_pb2.MonitorState)
        except state.FileNotFoundError:
            # TODO(ekasper): initialize state file with a setup script, so we
            # can raise with certainty when it's not found.
            logging.warning("Monitor state file not found, assuming first "
                            "run.")
        else:
            if not self.__state.HasField("verified_sth"):
                logging.warning("No verified monitor state, assuming first run.")

        # load compact merkle tree state from the monitor state
        self._verified_tree = merkle.CompactMerkleTree(hasher)
        self._unverified_tree = merkle.CompactMerkleTree(hasher)
        self._verified_tree.load(self.__state.verified_tree)
        self._unverified_tree.load(self.__state.unverified_tree)

    def __repr__(self):
        return "%r(%r, %r, %r)" % (self.__class__.__name__, self.__client,
                                   self.__verifier, self.__db)

    def __str__(self):
        return "%s(%s, %s, %s)" % (self.__class__.__name__, self.__client,
                                   self.__verifier, self.__db)

    def __update_state(self, new_state):
        """Update state and write to disk."""
        # save compact merkle tree state into the monitor state
        self._verified_tree.save(new_state.verified_tree)
        self._unverified_tree.save(new_state.unverified_tree)
        self.__state_keeper.write(new_state)
        self.__state = new_state
        logging.info("New state is %s" % new_state)

    @property
    def servername(self):
        return self.__client.servername

    @property
    def data_timestamp(self):
        """Timestamp of the latest verified data, in milliseconds since epoch.
        """
        return self.__state.verified_sth.timestamp

    def __fired_deferred(self, result):
        """"Create a fired deferred to indicate that the asynchronous operation
        should proceed immediately, when the result is already available."""
        fire = defer.Deferred()
        fire.callback(result)
        return fire

    def _set_pending_sth(self, new_sth):
        """Set pending_sth from new_sth, or just verified_sth if not bigger."""
        logging.info("STH verified, updating state.")
        if new_sth.tree_size < self.__state.verified_sth.tree_size:
            raise ValueError("pending size must be >= verified size")
        if new_sth.timestamp <= self.__state.verified_sth.timestamp:
            raise ValueError("pending time must be > verified time")
        new_state = client_pb2.MonitorState()
        new_state.CopyFrom(self.__state)
        if new_sth.tree_size > self.__state.verified_sth.tree_size:
            new_state.pending_sth.CopyFrom(new_sth)
        else:
            new_state.verified_sth.CopyFrom(new_sth)
        self.__update_state(new_state)
        return True

    def _set_verified_tree(self, new_tree):
        """Set verified_tree and maybe move pending_sth to verified_sth."""
        self._verified_tree = new_tree
        old_state = self.__state
        new_state = client_pb2.MonitorState()
        new_state.CopyFrom(self.__state)
        assert old_state.pending_sth.tree_size >= new_tree.tree_size
        if old_state.pending_sth.tree_size == new_tree.tree_size:
            # all pending entries retrieved
            # already did consistency checks so this should always be true
            assert (old_state.pending_sth.sha256_root_hash ==
                    self._verified_tree.root_hash())
            new_state.verified_sth.CopyFrom(old_state.pending_sth)
            new_state.ClearField("pending_sth")
        self.__update_state(new_state)
        # we just set new verified tree, so we report all changes
        self.__report.report()

    def _update_unverified_data(self, unverified_tree):
        self._unverified_tree = unverified_tree
        new_state = client_pb2.MonitorState()
        new_state.CopyFrom(self.__state)
        self.__update_state(new_state)

    def __get_audited_sth(self, sth, verify_status):
        audited_sth = client_pb2.AuditedSth()
        audited_sth.sth.CopyFrom(sth)
        audited_sth.audit.status = verify_status
        return audited_sth

    def __verify_consistency_callback(self, proof, old_sth, new_sth):
        self.__db.store_sth(self.servername,
                            self.__get_audited_sth(new_sth,
                                                   client_pb2.UNVERIFIED))
        try:
            logging.debug("got proof for (%s, %s): %s",
                old_sth.tree_size, new_sth.tree_size,
                map(lambda b: b[:8].encode("base64")[:-2] + "...", proof))
            self.__verifier.verify_sth_consistency(old_sth, new_sth, proof)
        except error.VerifyError as e:
            # catches both ConsistencyError and ProofError. when alerts are
            # implemented, only the former should trigger an immediate alert;
            # the latter may have innocent causes (e.g. data corruption,
            # software bug) so we could give it a chance to recover before
            # alerting.
            self.__db.store_sth(self.servername,
                                self.__get_audited_sth(new_sth,
                                                       client_pb2.VERIFY_ERROR))
            logging.error("Could not verify STH consistency: %s vs %s!!!\n%s" %
                          (old_sth, new_sth, e))
            raise
        else:
            self.__db.store_sth(self.servername,
                                self.__get_audited_sth(new_sth,
                                                       client_pb2.VERIFIED))

    def _verify_consistency(self, old_sth, new_sth):
        """Verifies that old STH is consistent with new STH.

        Returns: Deferred that fires with boolean indicating whether updating
        succeeded.

        Deferred raises: error.ConsistencyError if STHs were inconsistent"""
        proof = self.__client.get_sth_consistency(old_sth.tree_size,
                                                  new_sth.tree_size)
        proof.addCallback(self.__verify_consistency_callback, old_sth, new_sth)
        return proof

    def __update_sth_errback(self, failure):
        """Fired if there was network error or log server sent invalid
        response"""
        logging.error("get-sth from %s failed: %s" % (self.servername,
                                                     failure.getErrorMessage()))
        return False

    def __update_sth_verify_consistency_before_accepting_errback(self, failure):
        """Errback for verify_consistency method which is called before setting
        sth as verified. If STH was invalid appropriate error message is
        already logged, so we only want to return false as update_sth failed."""
        failure.trap(error.VerifyError)
        return False

    def __handle_old_sth_errback(self, failure, sth_response):
        failure.trap(error.VerifyError)
        logging.error("Received older STH which is older and inconsistent "
                      "with current verified STH: %s vs %s. Error: %s" %
                      (sth_response, self.__state.verified_sth, failure))

    def __handle_old_sth_callback(self, result, sth_response):
        logging.warning("Rejecting received "
                        "STH: timestamp is older than current verified "
                        "STH: %s vs %s " %
                        (sth_response, self.__state.verified_sth))

    def __update_sth_callback(self, sth_response):
        # If we got the same response as last time, do nothing.
        # If we got an older response than last time, make sure that it's
        # consistent with current verified STH and then return False.
        # (If older response is consistent then, there is nothing wrong
        # with the fact that we recieved older timestamp - the log could be
        # out of sync - but we should not rewind to older data.)
        #
        # The client should always return an STH but best eliminate the
        # None == None case explicitly by only shortcutting the verification
        # if we already have a verified STH.
        if self.__state.HasField("verified_sth"):
            if sth_response == self.__state.verified_sth:
                logging.info("Ignoring already-verified STH: %s" %
                             sth_response)
                return True
            elif (sth_response.timestamp <
                    self.__state.verified_sth.timestamp):
                d = self._verify_consistency(sth_response,
                                            self.__state.verified_sth)
                d.addCallback(self.__handle_old_sth_callback, sth_response)
                d.addErrback(self.__handle_old_sth_errback, sth_response)
                return False
        try:
            self.__verifier.verify_sth(sth_response)
        except (error.EncodingError, error.VerifyError):
            logging.error("Invalid STH: %s" % sth_response)
            return False

        # Verify consistency to catch the log trying to trick us
        # into rewinding the tree.
        d = self._verify_consistency(self.__state.verified_sth, sth_response)
        d.addCallback(lambda result: self._set_pending_sth(sth_response))
        d.addErrback(self.__update_sth_verify_consistency_before_accepting_errback)
        return d

    def _update_sth(self):
        """Get a new candidate STH. If update succeeds, stores the new STH as
        pending. Does nothing if there is already a pending
        STH.

        Returns: Deferred that fires with boolean indicating whether updating
        succeeded."""
        if self.__state.HasField("pending_sth"):
            return self.__fired_deferred(True)
        logging.info("Fetching new STH")
        sth_response = self.__client.get_sth()
        sth_response.addCallback(self.__update_sth_callback)
        sth_response.addErrback(self.__update_sth_errback)
        return sth_response

    def _compute_projected_sth_from_tree(self, tree, extra_leaves):
        partial_sth = client_pb2.SthResponse()
        old_size = tree.tree_size
        partial_sth.tree_size = old_size + len(extra_leaves)
        # we only want to check the hash, so just use a dummy timestamp
        # that looks valid so the temporal verifier doesn't complain
        partial_sth.timestamp = 0
        extra_raw_leaves = [leaf.leaf_input for leaf in extra_leaves]
        new_tree = tree.extended(extra_raw_leaves)
        partial_sth.sha256_root_hash = new_tree.root_hash()
        return partial_sth, new_tree


    def _compute_projected_sth(self, extra_leaves):
        """Compute a partial projected STH.

        Useful for when an intermediate STH is not directly available from the
        server, but you still want to do something with the root hash.

        Args:
            extra_leaves: Extra leaves present in the tree for the new STH, in
                the same order as in that tree.

        Returns:
            (partial_sth, new_tree)
            partial_sth: A partial STH with timestamp 0 and empty signature.
            new_tree: New CompactMerkleTree with the extra_leaves integrated.
        """
        return self._compute_projected_sth_from_tree(self._verified_tree,
                                                     extra_leaves)

    @staticmethod
    def __estimate_time(num_new_entries):
        if num_new_entries < 1000:
            return "a moment"
        elif num_new_entries < 1000000:
            return "a while"
        else:
            return "all night"

    def _fetch_entries_errback(self, e, consumer):
        logging.error("get-entries from %s failed: %s" %
                      (self.servername, e))
        consumer.done(None)
        return True


    def _scan_entries(self, entries):
        """Passes entries to certificate report.

        Args:
            entries: array of (entry_index, entry_response) tuples.
        """
        der_certs = []
        for entry_index, entry in entries:
            parsed_entry = entry_decoder.decode_entry(entry)
            ts_entry = parsed_entry.merkle_leaf.timestamped_entry
            if ts_entry.entry_type == client_pb2.X509_ENTRY:
                der_cert = ts_entry.asn1_cert
                der_chain = parsed_entry.extra_data.certificate_chain
            else:
                der_cert = (
                    parsed_entry.extra_data.precert_chain_entry.pre_certificate)
                der_chain = (
                    parsed_entry.extra_data.
                    precert_chain_entry.precertificate_chain)
            der_chain = der_chain[:]
            der_certs.append((entry_index, der_cert, der_chain,
                              ts_entry.entry_type))
        self.__report.scan_der_certs(der_certs)

    def _scan_entries_errback(self, e):
        logging.error("Failed to scan entries from %s: %s",
                      self.servername, e)
        self._update_unverified_data(self._verified_tree)
        return e

    class EntryConsumer(object):
        """Consumer for log_client.EntryProducer.

        When everything is consumed, consumed field fires a boolean indicating
        success of consuming.
        """
        def __init__(self, producer, monitor, pending_sth, verified_tree):
            self._producer = producer
            self._monitor = monitor
            self._pending_sth = pending_sth
            self._query_size = self._producer._end - self._producer._start + 1
            self._end = self._producer._end
            self._start = self._producer._start
            self._next_sequence_number = self._start
            #unverified_tree is tree that will be built during consumption
            self._next_sequence_number = self._producer._start
            self._unverified_tree = verified_tree
            self.consumed = defer.Deferred()
            self._fetched = 0

        def done(self, result):
            if not result:
                self.consumed.callback(False)
                return False
            self.result = result
            if result < self._query_size:
                logging.error("Failed to fetch all entries: expected tree size "
                              "%d vs retrieved tree size %d" %
                              (self._end + 1, self._next_sequence_number))
                self.consumed.callback(False)
                return False
            # check that the batch is consistent with the eventual pending_sth
            d = self._monitor._verify_new_tree(self._partial_sth, self._unverified_tree, result)
            d.chainDeferred(self.consumed)
            return True

        def consume(self, entry_batch):
            self._fetched += len(entry_batch)
            logging.info("Fetched %d entries (total: %d from %d)" %
                         (len(entry_batch), self._fetched, self._query_size))

            scan = threads.deferToThread(
                    self._monitor._scan_entries,
                    enumerate(entry_batch, self._next_sequence_number))
            scan.addErrback(self._monitor._scan_entries_errback)
            # calculate the hash for the latest fetched certs
            # TODO(ekasper): parse temporary data into permanent storage.
            self._partial_sth, self._unverified_tree = \
                    self._monitor._compute_projected_sth_from_tree(
                        self._unverified_tree, entry_batch)
            self._next_sequence_number += len(entry_batch)
            self._monitor._update_unverified_data(self._unverified_tree)
            return scan

    def _fetch_entries(self, start, end):
        """Fetches entries from the log.

        Returns: Deferred that fires with boolean indicating whether fetching
        suceeded"""
        num_new_entries = end - start + 1
        logging.info("Fetching %d new entries: this will take %s..." %
                     (num_new_entries,
                      self.__estimate_time(num_new_entries)))
        producer = self.__client.get_entries(start, end)
        consumer = Monitor.EntryConsumer(producer, self,
                                         self.__state.pending_sth,
                                         self._unverified_tree)
        d = producer.startProducing(consumer)
        d.addCallback(consumer.done)
        d.addErrback(self._fetch_entries_errback, consumer)
        return consumer.consumed

    def _update_entries(self):
        """Retrieve new entries according to the pending STH.

        Returns: Deferred that fires with boolean indicating whether updating
        succeeded.
        """
        if not self.__state.HasField("pending_sth"):
            return self.__fired_deferred(True)
        # Default is 0, which is what we want.
        wanted_entries = self.__state.pending_sth.tree_size
        last_parsed_size = self._unverified_tree.tree_size

        if wanted_entries > last_parsed_size:
            d = self._fetch_entries(last_parsed_size, wanted_entries-1)
        else:
            d = self._verify_entries()

        d.addErrback(self._update_entries_errback)
        return d

    def _update_entries_errback(self, failure):
        logging.error("Updating entries from %s failed: %s",
                      self.servername, failure)
        return False

    def _verify_entries(self):
        projected_sth, new_tree = self._compute_projected_sth_from_tree(
                self._unverified_tree, ())

        new_entries_count = new_tree.tree_size - self._verified_tree.tree_size

        return self._verify_new_tree(projected_sth, new_tree, new_entries_count)

    def _verify_new_tree(self, partial_sth, new_tree, new_entries_count):
        d = self._verify_consistency(partial_sth, self.__state.pending_sth)

        def set_verified_tree(result, new_tree, new_entries_count):
            logging.info("Verified %d entries", (new_entries_count))
            self._set_verified_tree(new_tree)
            return True

        d.addCallback(set_verified_tree, new_tree, new_entries_count)
        d.addErrback(self._verify_new_tree_errback)
        return d

    def _verify_new_tree_errback(self, failure):
            failure.trap(error.VerifyError)
            self._update_unverified_data(self._verified_tree)
            return False

    def _update_result(self, updates_result):
        if not updates_result:
            logging.error("Update failed")
        return updates_result

    def update(self):
        """Update log view. Returns True if the update succeeded, False if any
        error occurred."""
        logging.info("Starting update for %s" % self.servername)
        d = self._update_sth()
        d.addCallback(lambda sth_result: self._update_entries() if sth_result
                      else False)
        d.addCallback(self._update_result)
        return d





import copy

from ct.client import log_client
from ct.crypto import merkle
from ct.proto import client_pb2

DEFAULT_STH = client_pb2.SthResponse()
DEFAULT_STH.timestamp = 1234
DEFAULT_STH.tree_size = 1000
DEFAULT_STH.sha256_root_hash = "hash\x00"
DEFAULT_STH.tree_head_signature = "sig\xff"
DEFAULT_FAKE_PROOF = [(_c*32) for _c in "abc"]
DEFAULT_FAKE_ROOTS = [("cert-%d" % _i) for _i in range(4)]
DEFAULT_URI = "https://example.com"


class FakeHandlerBase(log_client.RequestHandler):
    """A fake request handler for generating responses locally."""

    def __init__(self, uri, entry_limit=0, tree_size=0):
        self._uri = uri
        self._entry_limit = entry_limit

        self._sth = copy.deepcopy(DEFAULT_STH)
        # Override with custom size
        if tree_size > 0:
            self._sth.tree_size = tree_size

    @classmethod
    def make_response(cls, code, reason, json_content=None):
        """Generate a response of the desired format."""
        raise NotImplementedError

    def get_sth(self):
        return self.make_response(200, "OK",
                                  json_content=sth_to_json(self._sth))

    def get_entries(self, start, end):
        end = min(end, self._sth.tree_size - 1)
        if start < 0 or end < 0 or start > end:
            return self.make_response(400, "Bad Request")

        if self._entry_limit > 0:
            end = min(start + self._entry_limit - 1, end)

        return self.make_response(200, "OK", json_content=entries_to_json(
            make_entries(start, end)))

    def get_sth_consistency(self, old_size, new_size):
        if not 0 <= old_size <= new_size <= self._sth.tree_size:
            return self.make_response(400, "Bad Request")

        return self.make_response(200, "OK",
                                  json_content=consistency_proof_to_json(
                                      DEFAULT_FAKE_PROOF))

    def get_roots(self):
        return self.make_response(200, "OK", json_content=roots_to_json(
            DEFAULT_FAKE_ROOTS))

    def get_entry_and_proof(self, leaf_index, tree_size):
        if (leaf_index >= tree_size or leaf_index < 0 or tree_size <= 0
            or tree_size > self._sth.tree_size):
            return self.make_response(400, "Bad Request")

        return self.make_response(200, "OK",
                                  json_content=entry_and_proof_to_json(
                                      make_entry(leaf_index),
                                      DEFAULT_FAKE_PROOF))

    def get_proof_by_hash(self, leaf_hash, tree_size):
        """If the hash is known, return a (fake) audit proof."""
        if (not leaf_hash or tree_size <= 0 or
            tree_size > self._sth.tree_size):
            return self.make_response(400, "Bad Request")

        hasher = merkle.TreeHasher()
        for i in range(self._sth.tree_size):
            entry = make_entry(i)
            if hasher.hash_leaf(entry.leaf_input) == leaf_hash:
                return self.make_response(200, "OK",
                                          json_content=proof_and_index_to_json(
                                              DEFAULT_FAKE_PROOF, i))

        # Not found
        return self.make_response(400, "Bad Request")

    def get_response(self, uri, params=None):
        """Generate a fake response."""
        if params is None:
            params = {}

        prefix = self._uri + "/ct/v1/"
        if not uri.startswith(prefix):
            return self.make_response(404, "Not Found")

        path = uri[len(prefix):]

        if path == "get-sth":
            return self.get_sth()

        elif path == "get-entries":
            start = int(params.get("start", -1))
            end = int(params.get("end", -1))
            return self.get_entries(start, end)

        elif path == "get-sth-consistency":
            old_size = int(params.get("first", -1))
            new_size = int(params.get("second", -1))
            return self.get_sth_consistency(old_size, new_size)

        elif path == "get-roots":
            return self.get_roots()

        elif path == "get-entry-and-proof":
            leaf_index = int(params.get("leaf_index", -1))
            tree_size = int(params.get("tree_size", -1))
            return self.get_entry_and_proof(leaf_index, tree_size)

        elif path == "get-proof-by-hash":
            leaf_hash = params.get("hash", "").decode("base64")
            tree_size = int(params.get("tree_size", -1))
            return self.get_proof_by_hash(leaf_hash, tree_size)

        else:
            # Bad path
            return self.make_response(404, "Not Found")


def make_entry(leaf_index):
    entry = client_pb2.EntryResponse()
    entry.leaf_input = "leaf_input-%d" % leaf_index
    entry.extra_data = "extra_data-%d" % leaf_index
    return entry


def make_entries(start, end):
    entries = []
    for i in range(start, end+1):
        entries.append(make_entry(i))
    return entries


def verify_entries(entries, start, end):
    if end - start + 1 != len(entries):
        return False
    for i in range(start, end+1):
        if make_entry(i) != entries[i]:
            return False
    return True


def sth_to_json(sth):
    return {"timestamp": sth.timestamp, "tree_size": sth.tree_size,
            "sha256_root_hash": sth.sha256_root_hash.encode("base64"),
            "tree_head_signature": sth.tree_head_signature.encode("base64")}


def entries_to_json(entries):
    return {"entries": [{"leaf_input": entry.leaf_input.encode("base64"),
                         "extra_data": entry.extra_data.encode("base64")}
                        for entry in entries]}


def consistency_proof_to_json(hashes):
    return {"consistency": [h.encode("base64") for h in hashes]}


def roots_to_json(roots):
    return {"certificates": [r.encode("base64") for r in roots]}


def entry_and_proof_to_json(entry, proof):
    return {"leaf_input": entry.leaf_input.encode("base64"),
            "extra_data": entry.extra_data.encode("base64"),
            "audit_path": [h.encode("base64") for h in proof]}


def proof_and_index_to_json(proof, leaf_index):
    return {"leaf_index": leaf_index,
            "audit_path": [h.encode("base64") for h in proof]}



#!/usr/bin/env trial
import copy
import difflib
import gflags
import logging
import mock
import os
import sys

from ct.client import log_client
from ct.client.db import sqlite_connection as sqlitecon
from ct.client.db import sqlite_log_db
from ct.client import state
from ct.client import monitor
from ct.crypto import error
from ct.crypto import merkle
from ct.proto import client_pb2
from twisted.internet import defer
from twisted.trial import unittest
from twisted.web import iweb
from zope.interface import implements

FLAGS = gflags.FLAGS

#TODO(ekasper) to make this setup common to all tests
gflags.DEFINE_bool("verbose_tests", False, "Print test logs")


def dummy_compute_projected_sth(old_sth):
    sth = client_pb2.SthResponse()
    sth.timestamp = old_sth.timestamp
    sth.tree_size = size = old_sth.tree_size
    tree = merkle.CompactMerkleTree(
        merkle.TreeHasher(), size, ["a"] * merkle.count_bits_set(size))
    f = mock.Mock(return_value=(sth, tree))
    f.dummy_sth = sth
    f.dummy_tree = tree
    old_sth.sha256_root_hash = tree.root_hash()
    return f

# TODO(robpercival): This is a relatively complicated fake, and may hide subtle
# bugs in how the Monitor interacts with the real EntryProducer. Using the real
# EntryProducer with a FakeAgent, as async_log_client_test does, may be an
# improvement.
class FakeEntryProducer(object):
    def __init__(self, start, end, batch_size=None, throw=None):
        self._start = start
        self._end = end
        self._real_start = start
        self._real_end = end
        self.throw = throw
        self.batch_size = batch_size if batch_size else end - start + 1
        self.stop = False

    @defer.deferredGenerator
    def produce(self):
        if self.throw:
            raise self.throw
        for i in range(self._start, self._end, self.batch_size):
            entries = []
            for j in range(i, min(i + self.batch_size, self._end)):
                entry = client_pb2.EntryResponse()
                entry.leaf_input = "leaf_input-%d" % j
                entry.extra_data = "extra_data-%d" % j
                entries.append(entry)
            d = self.consumer.consume(entries)
            wfd = defer.waitForDeferred(d)
            yield wfd
            wfd.getResult()
            if self.stop:
                break

        if not self.stop:
            self.done.callback(self._end - self._start + 1)

    def startProducing(self, consumer):
        self.stop = False
        self._start = self._real_start
        self._end = self._real_end
        self.consumer = consumer
        self.done = defer.Deferred()
        d = self.produce()
        d.addErrback(self.stopProducing)
        return self.done

    def change_range_after_start(self, start, end):
        """Changes query interval exactly when startProducing is ran.

        EntryConsumer in Monitor uses Producer interval, so in one of the tests
        we have to be able to change that interval when producing is started,
        but after consumer is created."""
        self._real_start = start
        self._real_end = end

    def stopProducing(self, failure=None):
        self.stop = True
        if failure:
            self.done.errback(failure)


class FakeLogClient(object):
    def __init__(self, sth, servername="log_server", batch_size=None,
                 get_entries_throw=None):
        self.servername = servername
        self.sth = sth
        self.batch_size = batch_size
        self.get_entries_throw = get_entries_throw

    def get_sth(self):
        d = defer.Deferred()
        d.callback(self.sth)
        return d

    def get_entries(self, start, end):
        return FakeEntryProducer(start, end, self.batch_size,
                                 self.get_entries_throw)

    def get_sth_consistency(self, old_tree, new_tree):
        d = defer.Deferred()
        d.callback([])
        return d

class InMemoryStateKeeper(object):
    def __init__(self, state=None):
        self.state = state
    def write(self, state):
        self.state = state
    def read(self, state_type):
        if not self.state:
            raise state.FileNotFoundError("Boom!")
        return_state = state_type()
        return_state.CopyFrom(self.state)
        return return_state

class MonitorTest(unittest.TestCase):
    _DEFAULT_STH = client_pb2.SthResponse()
    _DEFAULT_STH.timestamp = 2000
    _DEFAULT_STH.tree_size = 10
    _DEFAULT_STH.tree_head_signature = "sig"
    _DEFAULT_STH_compute_projected = dummy_compute_projected_sth(_DEFAULT_STH)

    _NEW_STH = client_pb2.SthResponse()
    _NEW_STH.timestamp = 3000
    _NEW_STH.tree_size = _DEFAULT_STH.tree_size + 10
    _NEW_STH.tree_head_signature = "sig2"
    _NEW_STH_compute_projected = dummy_compute_projected_sth(_NEW_STH)

    _DEFAULT_STATE = client_pb2.MonitorState()
    _DEFAULT_STATE.verified_sth.CopyFrom(_DEFAULT_STH)
    _DEFAULT_STH_compute_projected.dummy_tree.save(
            _DEFAULT_STATE.unverified_tree)
    _DEFAULT_STH_compute_projected.dummy_tree.save(
            _DEFAULT_STATE.verified_tree)

    def setUp(self):
        if not FLAGS.verbose_tests:
          logging.disable(logging.CRITICAL)
        self.db = sqlite_log_db.SQLiteLogDB(
            sqlitecon.SQLiteConnectionManager(":memory:", keepalive=True))
        # We can't simply use DB in memory with keepalive True, because different
        # thread is writing to the database which results in an sqlite exception.
        self.cert_db = mock.MagicMock()

        self.state_keeper = InMemoryStateKeeper(copy.deepcopy(self._DEFAULT_STATE))
        self.verifier = mock.Mock()
        self.hasher = merkle.TreeHasher()

        # Make sure the DB knows about the default log server.
        log = client_pb2.CtLogMetadata()
        log.log_server = "log_server"
        self.db.add_log(log)

    def verify_state(self, expected_state):
        if self.state_keeper.state != expected_state:
            state_diff = difflib.unified_diff(
                    str(expected_state).splitlines(),
                    str(self.state_keeper.state).splitlines(),
                    fromfile="expected", tofile="actual", lineterm="", n=5)

            raise unittest.FailTest("State is incorrect\n" +
                                    "\n".join(state_diff))

    def verify_tmp_data(self, start, end):
        # TODO: we are no longer using the temp db
        # all the callsites should be updated to test the main db instead
        pass

    def create_monitor(self, client, skip_scan_entry=True):
        m = monitor.Monitor(client, self.verifier, self.hasher, self.db,
                            self.cert_db, 7, self.state_keeper)
        if m:
            m._scan_entries = mock.Mock()
        return m

    def check_db_state_after_successful_updates(self, number_of_updates):
        audited_sths = list(self.db.scan_latest_sth_range("log_server"))
        for index, audited_sth in enumerate(audited_sths):
            if index % 2 != 0:
                self.assertEqual(client_pb2.UNVERIFIED,
                                 audited_sth.audit.status)
            else:
                self.assertEqual(client_pb2.VERIFIED,
                                 audited_sth.audit.status)
        self.assertEqual(len(audited_sths), number_of_updates * 2)

    def test_update(self):
        client = FakeLogClient(self._NEW_STH)

        m = self.create_monitor(client)
        m._compute_projected_sth_from_tree = self._NEW_STH_compute_projected
        def check_state(result):
            # Check that we wrote the state...
            expected_state = client_pb2.MonitorState()
            expected_state.verified_sth.CopyFrom(self._NEW_STH)
            m._compute_projected_sth_from_tree.dummy_tree.save(
                    expected_state.verified_tree)
            m._compute_projected_sth_from_tree.dummy_tree.save(
                    expected_state.unverified_tree)
            self.verify_state(expected_state)

            self.verify_tmp_data(self._DEFAULT_STH.tree_size,
                                 self._NEW_STH.tree_size-1)
            self.check_db_state_after_successful_updates(1)
            for audited_sth in self.db.scan_latest_sth_range(m.servername):
                self.assertEqual(self._NEW_STH, audited_sth.sth)

        return m.update().addCallback(self.assertTrue).addCallback(check_state)

    def test_first_update(self):
        client = FakeLogClient(self._DEFAULT_STH)

        self.state_keeper.state = None
        m = self.create_monitor(client)
        m._compute_projected_sth_from_tree = self._DEFAULT_STH_compute_projected
        def check_state(result):
            # Check that we wrote the state...
            self.verify_state(self._DEFAULT_STATE)

            self.verify_tmp_data(0, self._DEFAULT_STH.tree_size-1)
            self.check_db_state_after_successful_updates(1)
            for audited_sth in self.db.scan_latest_sth_range(m.servername):
                self.assertEqual(self._DEFAULT_STH, audited_sth.sth)

        d = m.update().addCallback(self.assertTrue
                                   ).addCallback(check_state)
        return d

    def test_update_no_new_entries(self):
        client = FakeLogClient(self._DEFAULT_STH)

        m = self.create_monitor(client)
        d = m.update()
        d.addCallback(self.assertTrue)

        def check_state(result):
            # Check that we kept the state...
            self.verify_state(self._DEFAULT_STATE)

            # ...and wrote no entries.
            self.check_db_state_after_successful_updates(0)
        d.addCallback(check_state)
        return d

    def test_update_recovery(self):
        client = FakeLogClient(self._NEW_STH)

        # Setup initial state to be as though an update had failed part way
        # through.
        initial_state = copy.deepcopy(self._DEFAULT_STATE)
        initial_state.pending_sth.CopyFrom(self._NEW_STH)
        self._NEW_STH_compute_projected.dummy_tree.save(
                initial_state.unverified_tree)
        self.state_keeper.write(initial_state)

        m = self.create_monitor(client)
        m._compute_projected_sth_from_tree = self._NEW_STH_compute_projected

        d = m.update()
        d.addCallback(self.assertTrue)

        def check_state(result):
            # Check that we wrote the state...
            expected_state = copy.deepcopy(initial_state)
            expected_state.ClearField("pending_sth")
            expected_state.verified_sth.CopyFrom(self._NEW_STH)
            m._compute_projected_sth_from_tree.dummy_tree.save(
                    expected_state.verified_tree)
            m._compute_projected_sth_from_tree.dummy_tree.save(
                    expected_state.unverified_tree)
            self.verify_state(expected_state)

            self.check_db_state_after_successful_updates(1)
            for audited_sth in self.db.scan_latest_sth_range(m.servername):
                self.assertEqual(self._NEW_STH, audited_sth.sth)
        d.addCallback(check_state)
        return d

    def test_update_rolls_back_unverified_tree_on_scan_error(self):
        client = FakeLogClient(self._NEW_STH)

        m = self.create_monitor(client)
        m._compute_projected_sth_from_tree = self._NEW_STH_compute_projected
        m._scan_entries = mock.Mock(side_effect=ValueError("Boom!"))

        def check_state(result):
            # The changes to the unverified tree should have been discarded,
            # so that entries are re-fetched and re-consumed next time.
            expected_state = copy.deepcopy(self._DEFAULT_STATE)
            expected_state.pending_sth.CopyFrom(self._NEW_STH)
            self.verify_state(expected_state)
            # The new STH should have been verified prior to the error.
            audited_sths = list(self.db.scan_latest_sth_range(m.servername))
            self.assertEqual(len(audited_sths), 2)
            self.assertEqual(audited_sths[0].audit.status, client_pb2.VERIFIED)
            self.assertEqual(audited_sths[1].audit.status, client_pb2.UNVERIFIED)

        return m.update().addCallback(self.assertFalse).addCallback(check_state)

    def test_update_call_sequence(self):
        # Test that update calls update_sth and update_entries in sequence,
        # and bails on first error, so we can test each of them separately.
        # Each of these functions checks if functions were properly called
        # and runs step in sequence of updates.
        def check_calls_sth_fails(result):
            m._update_sth.assert_called_once_with()
            m._update_entries.assert_called_once_with()

            m._update_sth.reset_mock()
            m._update_entries.reset_mock()
            m._update_sth.return_value = copy.deepcopy(d_false)
            return m.update().addCallback(self.assertFalse)

        def check_calls_entries_fail(result):
            m._update_sth.assert_called_once_with()
            self.assertFalse(m._update_entries.called)

            m._update_sth.reset_mock()
            m._update_entries.reset_mock()
            m._update_sth.return_value = copy.deepcopy(d_true)
            m._update_entries.return_value = copy.deepcopy(d_false)
            return m.update().addCallback(self.assertFalse)

        def check_calls_assert_last_calls(result):
            m._update_sth.assert_called_once_with()
            m._update_entries.assert_called_once_with()

        client = FakeLogClient(self._DEFAULT_STH)

        m = self.create_monitor(client)
        d_true = defer.Deferred()
        d_true.callback(True)
        d_false = defer.Deferred()
        d_false.callback(False)
        #check regular correct update
        m._update_sth = mock.Mock(return_value=copy.deepcopy(d_true))
        m._update_entries = mock.Mock(return_value=copy.deepcopy(d_true))
        d = m.update().addCallback(self.assertTrue)
        d.addCallback(check_calls_sth_fails)
        d.addCallback(check_calls_entries_fail)
        d.addCallback(check_calls_assert_last_calls)
        return d

    def test_update_sth(self):
        client = FakeLogClient(self._NEW_STH)

        m = self.create_monitor(client)

        def check_state(result):
            # Check that we updated the state.
            expected_state = copy.deepcopy(self._DEFAULT_STATE)
            expected_state.pending_sth.CopyFrom(self._NEW_STH)
            self.verify_state(expected_state)
            audited_sths = list(self.db.scan_latest_sth_range(m.servername))
            self.assertEqual(len(audited_sths), 2)
            self.assertEqual(audited_sths[0].audit.status, client_pb2.VERIFIED)
            self.assertEqual(audited_sths[1].audit.status, client_pb2.UNVERIFIED)

        return m._update_sth().addCallback(self.assertTrue
                                           ).addCallback(check_state)

    def test_update_sth_fails_for_invalid_sth(self):
        client = FakeLogClient(self._NEW_STH)
        self.verifier.verify_sth.side_effect = error.VerifyError("Boom!")

        m = self.create_monitor(client)
        def check_state(result):
            # Check that we kept the state.
            self.verify_state(self._DEFAULT_STATE)
            self.check_db_state_after_successful_updates(0)

        return m._update_sth().addCallback(self.assertFalse
                                           ).addCallback(check_state)

    def test_update_sth_fails_for_stale_sth(self):
        sth = client_pb2.SthResponse()
        sth.CopyFrom(self._DEFAULT_STH)
        sth.tree_size -= 1
        sth.timestamp -= 1
        client = FakeLogClient(sth)

        m = self.create_monitor(client)
        d = defer.Deferred()
        d.callback(True)
        m._verify_consistency = mock.Mock(return_value=d)
        def check_state(result):
            self.assertTrue(m._verify_consistency.called)
            args, _ = m._verify_consistency.call_args
            self.assertTrue(args[0].timestamp < args[1].timestamp)

            # Check that we kept the state.
            self.verify_state(self._DEFAULT_STATE)

        return m._update_sth().addCallback(self.assertFalse
                                           ).addCallback(check_state)

    def test_update_sth_fails_for_inconsistent_sth(self):
        client = FakeLogClient(self._NEW_STH)
        # The STH is in fact OK but fake failure.
        self.verifier.verify_sth_consistency.side_effect = (
            error.ConsistencyError("Boom!"))

        m = self.create_monitor(client)
        def check_state(result):
            # Check that we kept the state.
            self.verify_state(self._DEFAULT_STATE)
            audited_sths = list(self.db.scan_latest_sth_range(m.servername))
            self.assertEqual(len(audited_sths), 2)
            self.assertEqual(audited_sths[0].audit.status,
                             client_pb2.VERIFY_ERROR)
            self.assertEqual(audited_sths[1].audit.status,
                             client_pb2.UNVERIFIED)
            for audited_sth in audited_sths:
                self.assertEqual(self._DEFAULT_STH.sha256_root_hash,
                                 audited_sth.sth.sha256_root_hash)

        return m._update_sth().addCallback(self.assertFalse
                                           ).addCallback(check_state)

    def test_update_sth_fails_on_client_error(self):
        client = FakeLogClient(self._NEW_STH)
        def get_sth():
            return defer.maybeDeferred(mock.Mock(side_effect=log_client.HTTPError("Boom!")))
        client.get_sth = get_sth
        m = self.create_monitor(client)
        def check_state(result):
            # Check that we kept the state.
            self.verify_state(self._DEFAULT_STATE)
            self.check_db_state_after_successful_updates(0)

        return m._update_sth().addCallback(self.assertFalse
                                           ).addCallback(check_state)


    def test_update_entries_fails_on_client_error(self):
        client = FakeLogClient(self._NEW_STH,
                               get_entries_throw=log_client.HTTPError("Boom!"))
        client.get_entries = mock.Mock(
                return_value=client.get_entries(0, self._NEW_STH.tree_size - 2))

        m = self.create_monitor(client)

        # Get the new STH, then try (and fail) to update entries
        d = m._update_sth().addCallback(self.assertTrue)
        d.addCallback(lambda x: m._update_entries()).addCallback(self.assertFalse)

        def check_state(result):
            # Check that we wrote no entries.
            expected_state = copy.deepcopy(self._DEFAULT_STATE)
            expected_state.pending_sth.CopyFrom(self._NEW_STH)
            self.verify_state(expected_state)
        d.addCallback(check_state)

        return d

    def test_update_entries_fails_not_enough_entries(self):
        client = FakeLogClient(self._NEW_STH)
        faker_fake_entry_producer = FakeEntryProducer(0,
                                                      self._NEW_STH.tree_size)
        faker_fake_entry_producer.change_range_after_start(0, 5)
        client.get_entries = mock.Mock(
                return_value=faker_fake_entry_producer)

        m = self.create_monitor(client)
        m._compute_projected_sth = self._NEW_STH_compute_projected
        # Get the new STH first.
        return m._update_sth().addCallback(self.assertTrue).addCallback(
                lambda x: m._update_entries().addCallback(self.assertFalse))

    def test_update_entries_fails_in_the_middle(self):
        client = FakeLogClient(self._NEW_STH)
        faker_fake_entry_producer = FakeEntryProducer(
                self._DEFAULT_STH.tree_size,
                self._NEW_STH.tree_size)
        faker_fake_entry_producer.change_range_after_start(
            self._DEFAULT_STH.tree_size, self._NEW_STH.tree_size - 5)
        client.get_entries = mock.Mock(return_value=faker_fake_entry_producer)

        m = self.create_monitor(client)
        m._compute_projected_sth = self._NEW_STH_compute_projected
        fake_fetch = mock.MagicMock()
        def try_again_with_all_entries(_):
            m._fetch_entries = fake_fetch
            return m._update_entries()
        # Get the new STH first.
        return m._update_sth().addCallback(self.assertTrue).addCallback(
                lambda _: m._update_entries().addCallback(self.assertFalse)
                ).addCallback(try_again_with_all_entries).addCallback(lambda _:
                    fake_fetch.assert_called_once_with(15, 19))

if __name__ == "__main__" or __name__ == "ct.client.monitor_test":
    sys.argv = FLAGS(sys.argv)



import sqlite3
import gflags

from ct.client.db import cert_db
from ct.client.db import cert_desc

FLAGS = gflags.FLAGS

gflags.DEFINE_bool("cert_db_sqlite_synchronous_write", True, "If set to true, "
                   "sqlite will use synchronous write.")

class SQLiteCertDB(cert_db.CertDB):
    def __init__(self, connection_manager):
        """Initialize the database and tables.
        Args:
            connection: an SQLiteConnectionManager object."""
        self.__mgr = connection_manager
        cert_repeated_field_tables = [
            ("issuer", [("type", "TEXT"), ("name", "TEXT"),]),
            ("subject", [("type", "TEXT"), ("name", "TEXT"),]),
            ("subject_alternative_names", [("type", "TEXT"),
                                           ("name", "TEXT"),]),
            # subject common names and dnsnames for easy lookup of given
            # domain name
            ("subject_names", [("name", "TEXT")]),
            ("root_issuer", [("type", "TEXT"), ("name", "TEXT")])]
        cert_single_field_tables = [("version", "INTEGER"),
                                    ("serial_number", "TEXT")]
        with self.__mgr.get_connection() as conn:
            # the |cert| BLOB is also unique but we don't force this as it would
            # create a superfluous index.
            conn.execute("CREATE TABLE IF NOT EXISTS certs("
                         "log INTEGER,"
                         "id INTEGER,"
                         "sha256_hash BLOB UNIQUE,"
                         "cert BLOB," +
                         ', '.join(['%s %s' % (column, type_) for column, type_
                                    in cert_single_field_tables]) +
                         ", PRIMARY KEY(log, id))")
            for entry in cert_repeated_field_tables:
                self.__create_table_for_field(conn, *entry)
            conn.execute("CREATE INDEX IF NOT EXISTS certs_by_subject "
                         "on subject_names(name)")
        self.__tables = (["logs", "certs"] +
                         [column for column, _ in cert_repeated_field_tables])

    @staticmethod
    def __create_table_for_field(conn, table_name, fields):
        """Helper method that creates table for given certificate field. Each
        row in that table refers to some certificate in certs table.
        Args:
            table_name:   name of the table
            fields:       iterable of (column_name, type) tuples"""
        conn.execute("CREATE TABLE IF NOT EXISTS {table_name}("
                     "log INTEGER, cert_id INTEGER,"
                     "{fields},"
                     "FOREIGN KEY(log, cert_id) REFERENCES certs(log, id))"
                     .format(table_name=table_name,
                             fields=','.join(
                                     ["%s %s" % field for field in fields])))

    def __repr__(self):
        return "%r(db: %r)" % (self.__class__.__name__, self.__db)

    def __str__(self):
        return "%s(db: %s, tables: %s): " % (self.__class__.__name__, self.__db,
                                             self.__tables)


    @staticmethod
    def __compare_processed_names(prefix, name):
        return prefix == name[:len(prefix)]

    def __store_cert(self, cert, index, log_key, cursor):
        if not FLAGS.cert_db_sqlite_synchronous_write:
            cursor.execute("PRAGMA synchronous = OFF")
        try:
            cursor.execute("INSERT INTO certs(log, id, sha256_hash, cert, "
                           "version, serial_number) VALUES(?, ?, ?, ?, ?, ?) ",
                           (log_key, index,
                            sqlite3.Binary(cert.sha256_hash),
                            sqlite3.Binary(cert.der),
                            cert.version,
                            cert.serial_number,))
        except sqlite3.IntegrityError:
            # cert already exists
            return
        for sub in cert.subject:
            cursor.execute("INSERT INTO subject(log, cert_id, type, name)"
                           "VALUES(?, ?, ?, ?)",
                           (log_key, index, sub.type, sub.value))
            if sub.type == "CN":
                cursor.execute("INSERT INTO subject_names(log, cert_id, name)"
                               "VALUES(?, ?, ?)",
                               (log_key, index, sub.value))

        for alt in cert.subject_alternative_names:
            cursor.execute("INSERT INTO subject_alternative_names(log, cert_id,"
                           "type, name) VALUES(?, ?, ?, ?)",
                           (log_key, index, alt.type, alt.value))
            if alt.type == "dNSName":
                cursor.execute("INSERT INTO subject_names(log, cert_id, name)"
                               "VALUES(?, ?, ?)",
                               (log_key, index, alt.value))

        for iss in cert.issuer:
            cursor.execute("INSERT INTO issuer(log, cert_id, type, name)"
                           "VALUES(?, ?, ?, ?)",
                           (log_key, index, iss.type, iss.value))

        for iss in cert.root_issuer:
            cursor.execute("INSERT INTO root_issuer(log, cert_id, type, name)"
                           "VALUES(?, ?, ?, ?)",
                           (log_key, index, iss.type, iss.value))

    def store_certs_desc(self, certs, log_key):
        """Store certificates using their descriptions.

        Args:
            certs:         iterable of (CertificateDescription, index) tuples
            log_key:       log id in LogDB"""
        with self.__mgr.get_connection() as conn:
            cursor = conn.cursor()
            for cert in certs:
                self.__store_cert(cert[0], cert[1], log_key, cursor)

    def store_cert_desc(self, cert, index, log_key):
        """Store a certificate using its description.

        Args:
            cert:          CertificateDescription
            index:         position in log
            log_key:       log id in LogDB"""
        self.store_certs_desc([(cert, index)], log_key)

    def get_cert_by_sha256_hash(self, cert_sha256_hash):
        """Fetch a certificate with a matching SHA256 hash
        Args:
            cert_sha256_hash: the SHA256 hash of the certificate
        Returns:
            A DER-encoded certificate, or None if the cert is not found."""
        with self.__mgr.get_connection() as conn:
            res = conn.execute("SELECT cert FROM certs WHERE sha256_hash == ?",
                               (sqlite3.Binary(cert_sha256_hash),))
            try:
                return str(res.next()["cert"])
            except StopIteration:
                pass

    def scan_certs(self, limit=0):
        """Scan all certificates.
        Args:
            limit: maximum number of entries to yield. Default is no limit.
        Yields:
            DER-encoded certificates."""
        sql_limit = -1 if not limit else limit
        with self.__mgr.get_connection() as conn:
            for row in conn.execute(
                "SELECT cert FROM certs LIMIT ?", (sql_limit,)):
                yield str(row["cert"])

    # RFC 2818 (HTTP over TLS) states that "names may contain the wildcard
    # character * which is considered to match any single domain name
    # component or component fragment."
    #
    # So theoretically a cert for www.*.com or www.e*.com is valid for
    # www.example.com (although common browsers reject overly broad certs).
    # This makes wildcard matching in index scans difficult.
    #
    # The subject index scan thus does not match wildcards: it is not intended
    # for fetching all certificates that may be deemed valid for a given domain.
    # Applications should define their own rules for detecting wildcard certs
    # and anything else of interest.
    def scan_certs_by_subject(self, subject_name, limit=0):
        """Scan certificates matching a subject name.
        Args:
            subject_name: a subject name, usually a domain. A scan for
                          example.com returns certificates for www.example.com,
                          *.example.com, test.mail.example.com, etc. Similarly
                          'com' can be used to look for all .com certificates.
                          Wildcards are treated as literal characters: a search
                          for *.example.com returns certificates for
                          *.example.com but not for mail.example.com and vice
                          versa.
                          Name may also be a common name rather than a DNS name,
                          e.g., "Trustworthy Certificate Authority".
            limit:        maximum number of entries to yield. Default is no
                          limit.
        Yields:
            DER-encoded certificates."""
        sql_limit = -1 if not limit else limit
        prefix = cert_desc.process_name(subject_name)
        with self.__mgr.get_connection() as conn:
            for row in conn.execute(
                "SELECT certs.cert as cert, subject_names.name as name "
                "FROM certs, subject_names WHERE name >= ? AND certs.id == "
                "subject_names.cert_id ORDER BY name ASC LIMIT ?",
                (".".join(prefix), sql_limit)):
                name = cert_desc.process_name(row["name"], reverse=False)
                if self.__compare_processed_names(prefix, name):
                    yield str(row["cert"])
                else:
                    break



#!/usr/bin/env python

from distutils.core import setup

setup(name='Google Certificate Transparency',
      version='0.9',
      description='Certificate Transparency python client and monitor library',
      url='https://github.com/google/certificate-transparency',
      packages=['ct', 'ct.crypto', 'ct.crypto.asn1', 'ct.client'],
     )



import logging
import sqlite3
import time

from ct.client.db import log_db
from ct.client.db import database
from ct.proto import client_pb2

class SQLiteLogDB(log_db.LogDB):
    def __init__(self, connection_manager):
        """Initialize the database and tables.
        Args:
            connection_manager: an SQLiteConnectionManager object."""
        self.__mgr = connection_manager

        with self.__mgr.get_connection() as conn:
            # TODO(ekasper): give users control of table names via flags so
            # we can explicitly avoid conflicts between database objects
            # sharing the same underlying SQLiteConnection.
            conn.execute("CREATE TABLE IF NOT EXISTS logs("
                         "id INTEGER PRIMARY KEY, log_server TEXT UNIQUE, "
                         "metadata BLOB)")
            conn.execute("CREATE TABLE IF NOT EXISTS sths(log_id INTEGER, "
                         "fetch_timestamp INTEGER,"
                         "timestamp INTEGER, sth_data BLOB, "
                         "audit_info BLOB,"
                         "UNIQUE(log_id, timestamp, sth_data, audit_info) ON "
                         "CONFLICT IGNORE,"
                         "FOREIGN KEY(log_id) REFERENCES logs(id))")
            conn.execute("CREATE INDEX IF NOT EXISTS sth_by_timestamp on sths("
                         "log_id, timestamp)")
        self.__tables = ["logs", "sths"]

    def __repr__(self):
        return "%r(db: %r)" % (self.__class__.__name__, self.__mgr)

    def __str__(self):
        return "%s(db: %s, tables: %s): " % (self.__class__.__name__,
                                             self.__mgr, self.__tables)

    def __encode_log_metadata(self, metadata):
        log_server = metadata.log_server
        local_metadata = client_pb2.CtLogMetadata()
        local_metadata.CopyFrom(metadata)
        local_metadata.ClearField("log_server")
        return log_server, sqlite3.Binary(local_metadata.SerializeToString())

    def __decode_log_metadata(self, log_server, serialized_metadata):
        metadata = client_pb2.CtLogMetadata()
        metadata.ParseFromString(serialized_metadata)
        metadata.log_server = log_server
        return metadata

    def add_log(self, metadata):
        log_server, serialized_metadata = self.__encode_log_metadata(
            metadata)
        with self.__mgr.get_connection() as conn:
            try:
                conn.execute("INSERT INTO logs(log_server, metadata) "
                             "VALUES(?, ?)", (log_server, serialized_metadata))
            except sqlite3.IntegrityError:
                logging.warning("Ignoring duplicate log server %s", log_server)

    def update_log(self, metadata):
        log_server, serialized_metadata = self.__encode_log_metadata(
            metadata)
        with self.__mgr.get_connection() as conn:
            conn.execute("INSERT OR REPLACE INTO logs(id, log_server, "
                         "metadata) VALUES((SELECT id FROM logs WHERE "
                         "log_server = ?), ?, ?) ", (log_server, log_server,
                                                     serialized_metadata))

    def logs(self):
        with self.__mgr.get_connection() as conn:
            for log_server, metadata in conn.execute(
                "SELECT log_server, metadata FROM logs"):
                yield self.__decode_log_metadata(log_server, metadata)

    def _get_log_id(self, conn, log_server):
        res = conn.execute("SELECT id FROM logs WHERE log_server = ?",
                           (log_server,))
        try:
            log_id = res.next()
        except StopIteration:
            raise database.KeyError("Unknown log server: %s", log_server)
        return log_id[0]

    def get_log_id(self, log_server):
        with self.__mgr.get_connection() as conn:
            return self._get_log_id(conn, log_server)

    def __encode_sth(self, audited_sth):
        timestamp = audited_sth.sth.timestamp
        sth = client_pb2.SthResponse()
        sth.CopyFrom(audited_sth.sth)
        sth.ClearField("timestamp")
        audit = client_pb2.AuditInfo()
        audit.CopyFrom(audited_sth.audit)
        return (timestamp, sqlite3.Binary(sth.SerializeToString()),
                sqlite3.Binary(audit.SerializeToString()))

    def __decode_sth(self, sth_row):
        _, _, timestamp, serialized_sth, serialized_audit = sth_row
        audited_sth = client_pb2.AuditedSth()
        audited_sth.sth.ParseFromString(serialized_sth)
        audited_sth.sth.timestamp = timestamp
        audited_sth.audit.ParseFromString(serialized_audit)
        return audited_sth

    # This ignores a duplicate STH even if the audit data differs.
    # TODO(ekasper): add an update method for updating audit data, as needed.
    def store_sth(self, log_server, audited_sth):
        """Store the STH in the database.
        Will store the STH with a unique ID unless an exact copy already exists.
        Note: the fetch_timestamp is time of calling this function, not actual
        fetching timestamp.

        Args:
            log_server: the server name, i.e., the <log_server> path prefix
            audited_sth: a client_pb2.AuditedSth proto
        """
        timestamp, sth_data, audit_info = self.__encode_sth(audited_sth)
        with self.__mgr.get_connection() as conn:
            log_id = self._get_log_id(conn, log_server)
            conn.execute("INSERT INTO sths(log_id, fetch_timestamp, timestamp, "
                         "sth_data, audit_info) VALUES(?, ?, ?, ?, ?)",
                         (log_id, int(time.time()), timestamp, sth_data, audit_info))

    def get_latest_sth(self, log_server):
        row = None
        with self.__mgr.get_connection() as conn:
            log_id = self._get_log_id(conn, log_server)
            res = conn.execute("SELECT * FROM sths WHERE log_id = ? "
                               "ORDER BY timestamp DESC LIMIT 1", (log_id,))
            try:
                row = res.next()
            except StopIteration:
                pass
        if row is not None:
            return self.__decode_sth(row)

    def scan_latest_sth_range(self, log_server, start=0,
                              end=log_db.LogDB.timestamp_max, limit=0):
        sql_limit = -1 if not limit else limit
        with self.__mgr.get_connection() as conn:
            log_id = self._get_log_id(conn, log_server)
            for row in conn.execute(
                "SELECT * FROM sths WHERE log_id = ? "
                "AND timestamp >= ? AND timestamp <= ? ORDER BY timestamp DESC "
                "LIMIT ?", (log_id, start, end, sql_limit)):
                yield self.__decode_sth(row)



#!/usr/bin/env python

# Implement DNS inclusion proof checking, see [TBD].
#
# Unfortunately, getting at the SCTs in general is hard in Python, so this
# does not start with an SSL connection, but instead fetches a log entry by
# index and then verifies the proof over DNS.

# You will need to install DNSPython (http://www.dnspython.org/)

import base64
import dns.resolver
import hashlib
import json
import logging
import os
import sys
import urllib2

basepath = os.path.dirname(sys.argv[0])
sys.path.append(os.path.join(basepath, '../../../python'))
from ct.crypto import merkle, pem, verify
from ct.proto import client_pb2

class CTDNSLookup:
    def __init__(self, domain, verifier, resolver=None):
        self.verifier = verifier
        self.domain = domain
        self.resolver = resolver
        if not self.resolver:
            self.resolver = dns.resolver.get_default_resolver()

    def Get(self, name):
        logging.info('get %s', name)
        answers = self.resolver.query(name, 'TXT')
        assert answers.rdtype == dns.rdatatype.TXT
        return answers

    def GetOne(self, name):
        name += '.%s' % self.domain
        answers = self.Get(name)
        assert len(answers) == 1
        txt = answers[0]
        return ''.join(txt.strings)

    def GetSTH(self):
        sth_str = self.GetOne('sth')
        sth = client_pb2.SthResponse()
        parts = str(sth_str).split('.')
        sth.tree_size = int(parts[0])
        sth.timestamp = int(parts[1])
        sth.sha256_root_hash = base64.b64decode(parts[2])
        sth.tree_head_signature = base64.b64decode(parts[3])

        self.verifier.verify_sth(sth)

        return sth

    def GetEntry(self, level, index, size):
        return self.GetOne('%d.%d.%d.tree' % (level, index, size))

    def GetIndexFromHash(self, hash):
        return self.GetOne('%s.hash' % base64.b32encode(hash).rstrip('='))

if __name__ == '__main__':
    logging.basicConfig(level='INFO')

    index = sys.argv[1]

    keypem = ('-----BEGIN PUBLIC KEY-----\n'
              'MFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAEfahLEimAoz2t01p\n'
              '3uMziiLOl/fHTDM0YDOhBRuiBARsV4UvxG2LdNgoIGLrtCzWE0J\n'
              '5APC2em4JlvR8EEEFMoA==\n'
              '-----END PUBLIC KEY-----\n')
    logurl = 'http://ct.googleapis.com/pilot';
    logdns = 'pilot.ct.googleapis.com'

    response = urllib2.urlopen('%s/ct/v1/get-entries?start=%s&end=%s'
                               %  (logurl, index, index))
    j = response.read()
    j = json.loads(j)
    leaf_input = j['entries'][0]['leaf_input']
    logging.info('leaf = %s', leaf_input)
    leaf = base64.b64decode(leaf_input)
    leaf_hash = hashlib.sha256(chr(0) + leaf).digest()

    keyinfo = verify.create_key_info_from_raw_key(pem.from_pem(keypem, 'PUBLIC KEY')[0])
    log_verifier = verify.LogVerifier(keyinfo)

    lookup = CTDNSLookup(logdns, log_verifier)
    sth = lookup.GetSTH()
    logging.info('sth = %s', sth)

    logging.info('hash = %s', base64.b64encode(leaf_hash))
    verifier = merkle.MerkleVerifier()
    index = int(index)
    audit_path = []
    prev = None
    apl = verifier.audit_path_length(index, sth.tree_size)
    for level in range(0, apl):
        h = lookup.GetEntry(level, index, sth.tree_size)
        logging.info('hash = %s', base64.b64encode(h))
        audit_path.append(h[:32])

        if prev:
            if level < apl - 6:
                assert prev[32:] == h[:-32]
            else:
                assert prev[32:] == h
        else:
            assert len(h) == 32 * min(7, apl)

        prev = h

    logging.info('path = %s', map(base64.b64encode, audit_path))

    assert verifier.verify_leaf_hash_inclusion(leaf_hash, index, audit_path,
                                               sth)

    hash_info = lookup.GetIndexFromHash(leaf_hash)
    assert hash_info == str(index)



#!/usr/bin/env python
"""Compress the hashes list using Golomb coding."""

import math
import sys

from bitstring import BitArray
from bitstring import Bits
from bitstring import BitStream

import gflags

FLAGS = gflags.FLAGS

gflags.DEFINE_integer("hash_length", 8, "Length of each hash in bytes.")
gflags.DEFINE_integer("two_power", 47, "Power of 2 for M (M=2**two_power).")

def read_hashes(from_file, hash_length):
    """Reads a list of sorted hashes from a file."""
    with open(from_file, "rb") as f:
        raw_hashes = f.read()
        return [raw_hashes[i * hash_length:(i + 1) * hash_length]
                for i in range(len(raw_hashes) // hash_length)]


def golomb_encode(hashes_list, hash_length, M):
    """Given a sorted list of fixed-size values, compress it by
    using Golomb coding to represent the difference between the values."""
    hash_len_bits = hash_length * 8
    # Must be sorted for deltas to be small and easily compressable.
    assert sorted(hashes_list) == hashes_list
    # Must not contain duplicates.
    assert len(hashes_list) == len(set(hashes_list))
    # M is the tunable parameter.
    m_bits = int(math.log(M, 2))
    # Make sure that M is a power of 2.
    assert M > 0 and not (M & (M - 1))

    # First item in the output bit array is the first hash value.
    outarray = BitArray(bytes = hashes_list[0], length=hash_len_bits)

    # Set to true when the diff value / M == 0.
    # If no such value exists then the chosen M is too small, so warn.
    min_is_zero = False
    prev = BitArray(bytes = hashes_list[0], length=hash_len_bits)
    for curr_hash in hashes_list[1:]:
        curr = BitArray(bytes=curr_hash, length=hash_len_bits)
        N = curr.uint - prev.uint
        q = int(math.floor(N / M))
        r = N % M
        # Unary-encode q.
        if q == 0:
            outarray.append(Bits(bin='0b0'))
            min_is_zero = True
        else:
            outarray.append(Bits(bin=bin(2**q - 1) + '0'))

        # Write r using plain binary representation.
        outarray.append(Bits(uint=r, length=m_bits))
        prev = curr

    if not min_is_zero:
        print "Inefficient encoding: Minimum is not zero."
    return outarray.tobytes()


def uncompress_golomb_coding(coded_bytes, hash_length, M):
    """Given a bytstream produced using golomb_coded_bytes, uncompress it."""
    ret_list = []
    instream = BitStream(
        bytes=coded_bytes, length=len(coded_bytes) * 8)
    hash_len_bits = hash_length * 8
    m_bits = int(math.log(M, 2))
    # First item is a full hash value.
    prev = instream.read("bits:%d" % hash_len_bits)
    ret_list.append(prev.tobytes())

    while (instream.bitpos + m_bits) <= instream.length:
        # Read Unary-encoded value.
        read_prefix = 0
        curr_bit = instream.read("uint:1")
        while curr_bit == 1:
            read_prefix += 1
            curr_bit = instream.read("uint:1")
        assert curr_bit == 0

        # Read r, assuming M bits were used to represent it.
        r = instream.read("uint:%d" % m_bits)
        curr_diff = read_prefix * M + r
        curr_value_int = prev.uint + curr_diff
        curr_value = Bits(uint=curr_value_int, length=hash_len_bits)
        ret_list.append(curr_value.tobytes())
        prev = curr_value

    return ret_list


def main(input_file, output_file):
    """Reads and compresses the hashes."""
    hashes = read_hashes(input_file, FLAGS.hash_length)
    hashes.sort()

    golomb_coded_bytes = golomb_encode(
        hashes, FLAGS.hash_length, 2**FLAGS.two_power)

    print "With M=2**%d, Golomb-coded data size is %d, compression ratio %f" % (
            FLAGS.two_power,
            len(golomb_coded_bytes),
            len(golomb_coded_bytes) / float(len(hashes) * FLAGS.hash_length))

    with open(output_file, 'wb') as f:
        f.write(golomb_coded_bytes)

    uncompressed_hashes = uncompress_golomb_coding(
        golomb_coded_bytes, FLAGS.hash_length, 2**FLAGS.two_power)

    print "Original hashes: %d  Uncompressed: %d" % (
        len(hashes), len(uncompressed_hashes))
    assert uncompressed_hashes == hashes

if __name__ == '__main__':
    sys.argv = FLAGS(sys.argv)
    if len(sys.argv) < 3:
        sys.stderr.write(
            "Usage: %s <input hashes file> <compressed output file>\n"
            "  <input hashes file> Is the truncated, uncompressed hashes "
            "list.\n"
            "  <compressed output file> is the output, Golomb-coded file.\n" %
            sys.argv[0])
        sys.exit(1)
    main(sys.argv[1], sys.argv[2])



import json, sys, cmd
import cPickle as pickle

from verifiable_base import VerifiableBase
from verifiable_log import VerifiableLog
from verifiable_map import VerifiableMap, recalc_tree_hash

# Example general purpose verifiable database
# Mutation opertions append to its log
# Its verifiable map then calls the callback (_apply_operation) to change the view.
class VerifiableDatabase(VerifiableBase):
  def __init__(self):
    VerifiableBase.__init__(self, VerifiableLog())

  # Private, call back for the underlying map when new entries are sequenced by the log
  def _apply_operation(self, idx, operation, map):
    op = json.loads(operation)
    if op['operation'] == 'set':
      map.put(str(op['key']), str(op['value']))
    elif op['operation'] == 'delete':
      map.put(str(op['key']), '')

  # Example database operation
  def set(self, key, value):
    self._log.append(json.dumps({'operation': 'set', 'key': key, 'value': value}))

  # Example database operation
  def delete(self, key):
    self._log.append(json.dumps({'operation': 'delete', 'key': key}))

  # Return a value for a key and given tree_size (as returned by get_tree_head)
  # Also returns proof
  def get(self, key, tree_size):
    val, proof = VerifiableBase.get(self, str(key), tree_size)
    val = str(val) if len(val) else None
    return val, proof

# Test right val is returned and inclusion proof checks out
def test(db, query, tree_size, exp_val):
  val, proof = db.get(query, tree_size)
  assert val == exp_val
  assert recalc_tree_hash(query, str(val) if val else '', proof) == db.get_tree_head(tree_size)['sha256_root_hash']


class ReplCmd(cmd.Cmd):
  def __init__(self):
    cmd.Cmd.__init__(self)
    self.prompt = '> '
    self.do_new()

  def do_sth(self, arg):
    try:
      if not len(arg.strip()):
        seq = None
      else:
        seq = int(arg)
    except:
      self.help_sth()
      return
    print self.db.get_tree_head(seq)

  def help_sth(self):
    print 'sth <integer> - Updates tree to sequence number and print STH. Leave blank for latest.'

  def do_new(self, arg=''):
    self.db = VerifiableDatabase()

  def help_new(self):
    print 'new - creates a new database, called by default upon launch'

  def do_save(self, arg):
    arg = arg.strip()
    if not len(arg):
      self.help_save()
      return
    pickle.dump(self.db, file(arg, 'wb'))

  def help_save(self):
    print 'save <path> - save state to a path'

  def do_load(self, arg):
    arg = arg.strip()
    if not len(arg):
      self.help_load()
      return
    self.db = pickle.load(file(arg, 'rb'))

  def help_load(self):
    print 'load <path> - load state from path'

  def do_set(self, arg):
    try:
      n, v = arg.split(' ')
    except:
      self.help_set()
      return
    n = n.strip()
    v = v.strip()
    self.db.set(n, v)
    self.do_get(n)

  def help_set(self):
    print 'set <key> <value> - set key (string) to the specified value (string)'

  def do_get(self, arg):
    try:
      n, v = arg.split(' ')
      n = n.strip()
      v = v.strip()
    except:
      n = arg.strip()
      v = self.db.get_tree_head(None)['tree_size']
    try:
      v = int(v)
    except:
      self.help_get()
      return
    try:
      val, proof = self.db.get(n, v)
    except ValueError:
      print 'Tree size does not exist.'
      return
    print 'Value:     ', val
    print 'Proof:     ', proof
    print 'Map hash:  ', self.db.get_tree_head(v)['sha256_root_hash']
    print 'Log hash:  ', self.db.get_tree_head(v)['log_tree_head']['sha256_root_hash']
    print 'Tree size: ', self.db.get_tree_head(v)['tree_size']

  def help_get(self):
    print 'get <key> <integer> - get value as of this sequence number.  Leave blank for latest.'

  def do_del(self, arg):
    n = arg.strip()
    self.db.delete(n)
    self.do_get(n)

  def help_del(self):
    print 'del <key> - delete key (string) from database'

  def do_dump(self, arg=''):
    try:
      if not len(arg.strip()):
        seq = None
      else:
        seq = int(arg)
    except:
      self.help_dump()
      return
    print 'Tree:'
    self.db.debug_dump(seq)

  def help_dump(self):
    print 'dump <integer> - dump the tree as of this sequence number.  Leave blank for latest.'

  def do_log(self, arg=''):
    for i, x in enumerate(self.db.get_log_entries(0, self.db.get_tree_head()['tree_size'] - 1)):
      print i, x

  def help_log(self):
    print 'log - dump all ops'


db = VerifiableDatabase()
db.set('foo', 'bar')
db.set('foo', 'baz')

db.delete('foo')
db.set('foo', 'bar')
db.get_tree_head()

test(db, 'foo', 0, None)
test(db, 'foo', 1, 'bar')
test(db, 'foo', 2, 'baz')
test(db, 'foo', 3, None)
test(db, 'foo', 4, 'bar')

if __name__ == '__main__':
  ReplCmd().cmdloop('Type "help" to get started.')



from ct.crypto import error
from ct.crypto import pem
from ct.proto import client_pb2

import Crypto.Hash.SHA256
import Crypto.PublicKey.RSA
import Crypto.Signature.PKCS1_v1_5

class RsaVerifier(object):
    """Verifies RSA signatures."""

    # The signature algorithm used for this public key.
    SIGNATURE_ALGORITHM = client_pb2.DigitallySigned.RSA
    # The hash algorithm used for this public key.
    HASH_ALGORITHM = client_pb2.DigitallySigned.SHA256

    # Markers to look for when reading a PEM-encoded RSA public key.
    __READ_MARKERS = ("PUBLIC KEY", "RSA PUBLIC KEY")
    # A marker to write when writing a PEM-encoded RSA public key.
    __WRITE_MARKER = "RSA PUBLIC KEY"

    def __init__(self, key_info):
        """Creates a verifier that uses a PEM-encoded RSA public key.

        Args:
        - key_info: KeyInfo protobuf message

        Raises:
        - PemError: If the key has an invalid encoding
        """
        if (key_info.type != client_pb2.KeyInfo.RSA):
            raise error.UnsupportedAlgorithmError(
                "Expected RSA key, but got key type %d" % key_info.type)

        # Will raise a PemError on invalid encoding
        self.__der, _ = pem.from_pem(key_info.pem_key, self.__READ_MARKERS)
        try:
            self.__key = Crypto.PublicKey.RSA.importKey(self.__der)
        except (ValueError, IndexError, TypeError) as e:
            raise error.EncodingError(e)

    def __repr__(self):
        return "%s(public key: %r)" % (self.__class__.__name__,
                                       pem.to_pem(self.__der,
                                                  self.__WRITE_MARKER))

    @error.returns_true_or_raises
    def verify(self, signature_input, signature):
        """Verifies the signature was created by the owner of the public key.

        Args:
        - signature_input: The data that was originally signed.
        - signature: An RSA SHA256 signature.

        Returns:
        - True if the signature verifies.

        Raises:
        - error.SignatureError: If the signature fails verification.
        """
        verifier = Crypto.Signature.PKCS1_v1_5.new(self.__key)
        sha256_hash = Crypto.Hash.SHA256.new(signature_input)

        if verifier.verify(sha256_hash, signature):
            return True
        else:
            raise error.SignatureError("Signature did not verify: %s",
                                       signature.encode("hex"))




import abc
import gflags
import hashlib
import logging
import multiprocessing
import sys
import threading
import traceback

from ct.client.db import cert_desc
from ct.crypto import cert
from ct.crypto import error
from ct.proto import certificate_pb2
from Queue import Queue

FLAGS = gflags.FLAGS

gflags.DEFINE_integer("reporter_workers", multiprocessing.cpu_count(),
                      "Number of subprocesses scanning certificates.")

gflags.DEFINE_integer("reporter_queue_size", 50,
                      "Size of entry queue in reporter")


def _scan_der_cert(der_certs):
    current = -1
    result = []
    for log_index, der_cert, der_chain, entry_type in der_certs:
        try:
            current = log_index
            certificate = None
            strict_failure = False
            try:
                certificate = cert.Certificate(der_cert)
            except error.Error as e:
                try:
                    certificate = cert.Certificate(der_cert, strict_der=False)
                except error.Error as e:
                    strict_failure = True
            if not strict_failure:
                desc = cert_desc.from_cert(certificate)
            else:
                desc = certificate_pb2.X509Description()
                desc.der = der_cert
                desc.sha256_hash = hashlib.sha256(der_cert).digest()

            desc.entry_type = entry_type
            root = None

            if der_chain:
                try:
                    issuer = cert.Certificate(der_chain[0], strict_der=False)
                except error.Error:
                    pass
                else:
                    desc.issuer_pk_sha256_hash = issuer.key_hash(hashfunc="sha256")

                try:
                    root = cert.Certificate(der_chain[-1], strict_der=False)
                except error.Error:
                    pass
            else:
                # No chain implies this is a root certificate.
                # Note that certificate may be None.
                root = certificate

            if root:
                for iss in [(type_.short_name, cert_desc.to_unicode(
                        '.'.join(cert_desc.process_name(value.human_readable()))))
                            for type_, value in root.issuer()]:
                    proto_iss = desc.root_issuer.add()
                    proto_iss.type, proto_iss.value = iss

            result.append((desc, log_index))
        except:
            batch_start_index, batch_end_index = (
                    der_certs[0][0], der_certs[-1][0])
            logging.exception(
                    "Error scanning certificate %d in batch <%d, %d> - it will "
                    "be excluded from the scan results",
                    current, batch_start_index, batch_end_index)

    return result


class CertificateReport(object):
    """Stores description of new entries between last verified STH and current."""
    __metaclass__ = abc.ABCMeta

    def __init__(self, queue_size=None):
        self.reset()
        self._jobs = Queue(queue_size or FLAGS.reporter_queue_size)
        self._pool = None
        self._writing_handler = None

    def _writing_handler_ready(self):
        return self._writing_handler and self._writing_handler.is_alive()

    @abc.abstractmethod
    def report(self):
        """Report stored changes and reset report."""
        if self._writing_handler_ready():
            self._jobs.join()
            self._jobs.put(None)
            self._writing_handler.join()
            self._writing_handler = None

    @abc.abstractmethod
    def _batch_scanned_callback(self, result):
        """Callback called after scanning der_certs passed to scan_der_certs."""

    @abc.abstractmethod
    def reset(self):
        """Clean up report."""

    def scan_der_certs(self, der_certs):
        """Scans certificates in der form, parsing them to produce
        X509Description for each.

        Args:
            der_certs: non empty array of
                       (log_index, der_cert, der_chain, entry_type) tuples.
        """
        if not self._pool:
            self._pool = multiprocessing.Pool(processes=FLAGS.reporter_workers)
        if not self._writing_handler_ready():
            self._writing_handler = threading.Thread(target=handle_writing,
                                                     args=(self._jobs, self))
            self._writing_handler.start()
        self._jobs.put(self._pool.apply_async(_scan_der_cert,
                                                 [der_certs]))


def handle_writing(queue, report):
    while True:
        try:
            scan_results = queue.get()
            # This check must be performed in the try block so task_done will
            # be invoked in the finally block regardless of the check results.
            if not scan_results:
                break
            report._batch_scanned_callback(scan_results.get())
        except:
            logging.exception("Error occurred during certificate scanning")
        finally:
            queue.task_done()



#!/usr/bin/env python

from ct.client import async_log_client
from twisted.internet import reactor
from twisted.web import client as twisted_client


def sth_callback(sth):
    print sth


def stop_callback(ignored):
    reactor.stop()


def get_sth():
    agent = twisted_client.Agent(reactor)
    client = async_log_client.AsyncLogClient(
            agent, "https://ct.googleapis.com/pilot")
    d = client.get_sth()
    # Print the STH on success.
    d.addCallback(sth_callback)
    # Stop the reactor whether we succeeded or not.
    d.addBoth(stop_callback)


if __name__ == "__main__":
    # Schedule the call.
    get_sth()
    # Start the event loop.
    reactor.run()



import abc

class TempDB(object):
    """Database interface for storing unverified CT log entry data."""
    @abc.abstractmethod
    def drop_entries(self):
        """Drop all entries."""

    @abc.abstractmethod
    def store_entries(self, entries):
        """Batch store log entries.
        Args:
            entries: an iterable of (entry_number, client_pb2.EntryResponse)
                     tuples
       """

    @abc.abstractmethod
    def scan_entries(self, start, end):
        """Retrieve log entries.
        Args:
            start: index of the first entry to retrieve.
            end: index of the last entry to retrieve.
        Yields:
            client_pb2.EntryResponse protos.
        Raises:
            KeyError: an entry with a sequence number in the range does not
                      exist.
        """



#!/usr/bin/env python
"""cert_util.py: X509 certificate parsing utility.

Usage:

  cert_util.py <command> [flags] [cert_file ...]

Known commands:

  print: print information about the certificates in given files

  Each file must contain either one or more PEM-encoded certificates,
  or a single DER certificate.

  For example:

  cert_util.py print cert.pem           - pretty-print the certificate(s)
  cert_util.py print c1.pem c2.pem      - pretty-print certificates from
                                          multiple files
  cert_util.py print cert.der           - both PEM and DER are accepted formats
                                          (use --filetype to force a format)
  cert_util.py print --debug cert.pem   - print full ASN.1 structure
  cert_util.py print --subject cert.pem - print the subject name
  cert_util.py print --issuer cert.pem  - print the issuer name
  cert_util.py print --fingerprint cert.pem
                                        - print the SHA-1 fingerprint
  cert_util.py print --fingerprint --digest="sha256" cert.pem
                                        - print the SHA-256 fingerprint
"""

import sys
from ct.crypto import cert
from ct.crypto import error
from ct.crypto import pem
from ct.crypto.asn1 import print_util
import gflags

FLAGS = gflags.FLAGS
gflags.DEFINE_bool("subject", False, "Print option: prints certificate subject")
gflags.DEFINE_bool("issuer", False, "Print option: prints certificate issuer")
gflags.DEFINE_bool("fingerprint", False, "Print option: prints certificate "
                   "fingerprint")
gflags.DEFINE_string("digest", "sha1", "Print option: fingerprint digest to use")
gflags.DEFINE_bool("debug", False,
                   "Print option: prints full ASN.1 debug information")
gflags.DEFINE_string("filetype", "", "Read option: specify an input file "
                     "format (pem or der). If no format is specified, the "
                     "parser attempts to detect the format automatically.")
gflags.RegisterValidator("filetype", lambda value: not value or
                         value.lower() in {"pem", "der"},
                         message="--filetype must be one of pem or der")


def print_cert(certificate):
    if not FLAGS.subject and not FLAGS.issuer and not FLAGS.fingerprint:
        if FLAGS.debug:
            print "%r" % certificate
        else:
            print certificate
    else:
        if FLAGS.subject:
            print "subject:\n%s" % certificate.print_subject_name()
        if FLAGS.issuer:
            print "issuer:\n%s" % certificate.print_issuer_name()
        if FLAGS.fingerprint:
            # Print in a format familiar from OpenSSL.
            print "%s fingerprint: %s\n" % (
                FLAGS.digest.upper(), print_util.bytes_to_hex(
                    certificate.fingerprint(hashfunc=FLAGS.digest)))


def print_certs(cert_file):
    """Print the certificates, or parts thereof, as specified by flags."""
    # If no format is specified, try PEM first, and automatically fall back
    # to DER. The advantage is that usage is more convenient; the disadvantage
    # is that error messages are less helpful because we don't know the expected
    # file format.
    printed = False
    if not FLAGS.filetype or FLAGS.filetype.lower() == "pem":
        if not FLAGS.filetype:
            print "Attempting to read PEM"

        try:
            for c in cert.certs_from_pem_file(cert_file, strict_der=False):
                print_cert(c)
                printed = True
        except pem.PemError as e:
            if not printed:
                # Immediate error
                print "File is not a valid PEM file: %s" % e
            else:
                exit_with_message("Error while scanning PEM blocks: %s" % e)
        except error.ASN1Error as e:
            exit_with_message("Bad DER encoding: %s" % e)

    if not printed and FLAGS.filetype.lower() != "pem":
        if not FLAGS.filetype:
            print "Attempting to read raw DER"
        try:
            print_cert(cert.Certificate.from_der_file(cert_file,
                                                      strict_der=False))
        except error.ASN1Error as e:
            exit_with_message("Failed to parse DER from %s" % cert_file)


def exit_with_message(error_message):
    print error_message
    print "Use --helpshort or --help to get help."
    sys.exit(1)


def main(argv):
    if len(argv) <= 1 or argv[1][0] == "-":
        # No command. Parse flags anyway to trigger help flags.
        try:
            argv = FLAGS(argv)
            exit_with_message("No command")
        except gflags.FlagsError as e:
            exit_with_message("Error parsing flags: %s" % e)

    argv = argv[1:]

    try:
        argv = FLAGS(argv)
    except gflags.FlagsError as e:
        exit_with_message("Error parsing flags: %s" % e)

    command, argv = argv[0], argv[1:]

    if command != "print":
        exit_with_message("Unknown command %s" % command)

    if not argv:
        exit_with_message("No certificate file given")
    for filename in argv:
        print_certs(filename)
    sys.exit(0)

if __name__ == "__main__":
    main(sys.argv)



#!/usr/bin/env python
"""Generates a list of hashes of EV certificates found in a log."""

import json
import os
import requests
import shutil
import StringIO
import sys
import tempfile
import urlparse
import zipfile

import generate_ev_whitelist
import golomb_code

import gflags

CRX_URL = (
    "https://clients2.googleusercontent.com/crx/blobs/QgAAAC6zw0qH2DJtnXe8Z7rUJP35Av6_uD2R0shIWqC2XwPmpwrDdH5TKyq_6MWRGfvCXV3f-8g220oUjAO-JvMYG2m5CSvsx6cHch05zQS7ZbtrAMZSmuWMzK-Yj7NpgCxtqGhyTFveAvi-IQ/oafdbfcohdcjandcenmccfopbeklnicp_main.platform.all.crx")

TRUNCATED_HASH_LENGTH = 8
GOLOMB_M_PARAMETER = 2 ** 47
NUM_FETCHING_PROCESSES = 40
# Map between a log's URL and the index of the cut-off point after which
# no new EV certificates will be whitelisted.
# These entries are basically the index of the last EV cert that was found on
# January 1st, when the EV whitelist was finalized.
# Ultimately this allows re-generation of the whitelist, excluding certificates
# that have recently expired, thus reducing its size.
LOGS_LIST = {
    "https://ct.googleapis.com/pilot":6082932,
    "https://ct.googleapis.com/aviator":5360225,
    "https://ct1.digicert-ct.com/log":1056,
    "https://log.certly.io": 4729,
    "https://ct.izenpe.com": 1,
}

FLAGS = gflags.FLAGS

gflags.DEFINE_string(
    "output_crx", "chrome_ev_whitelist_%d",
    "Output Chrome extension. "
    "This zip file should be uploaded to the Chrome web store.")

gflags.DEFINE_string("output_truncated_hashes", "raw_truncated_hashes",
                     "The truncated hashes, hex-encoded, before compression.")


class EVCRXHandler(object):
    """Produces updated EV whitelist Chrome extension."""
    def __init__(self, crx_url=CRX_URL):
        req = requests.get(crx_url)
        if not req.ok:
            raise Exception("Error while fetching %s: %d" %
                            crx_url, req.status_code)
        # Skip the CRX header.
        crx_zip_bytes = req.content[566:]
        sio = StringIO.StringIO(crx_zip_bytes)
        zipped_crx = zipfile.ZipFile(sio)
        unpacked_crx_dir = tempfile.mkdtemp(prefix="tmpcrx")
        zipped_crx.extractall(unpacked_crx_dir)
        self._crx_dir = unpacked_crx_dir

    def _read_manifest_json(self):
        """Reads and parses the extension's metadata from manifest.json"""
        with open(os.path.join(self._crx_dir, "manifest.json")) as manifest:
            return json.load(manifest)

    def _write_manifest_json(self, json_to_write):
        """Overwrites the extension's metadata with the provided json."""
        with open(os.path.join(self._crx_dir, "manifest.json"), "wb") as manifest:
            json.dump(json_to_write, manifest)

    def manifest_version(self):
        """Returns the version number from the extension's metadata."""
        manifest = self._read_manifest_json()
        return int(manifest["version"])

    def update_ev_whitelist(self, compressed_ev_whitelist):
        """Writes an updated EV whitelist into the extension.

        Each new EV whitelist must be uploaded with a version number greater
        than the current version, so increments the version number.
        """
        manifest = self._read_manifest_json()
        manifest["version"] = str(self.manifest_version() + 1)
        self._write_manifest_json(manifest)
        with open(
            os.path.join(
                self._crx_dir,
                "_platform_specific",
                "all",
                "ev_hashes_whitelist.bin"),
            "wb") as hashes_file:
            hashes_file.write(compressed_ev_whitelist)

    def pack_to(self, dest_file_template):
        """Packs the extesion to the provided destination.

        Note that the provided dest_file_template is a template string so
        the version number can be filled in.
        Returns the exact file name that was written.
        """
        fname = dest_file_template % self.manifest_version()
        shutil.make_archive(fname, "zip", self._crx_dir)
        return fname

    def cleanup(self):
        """Deletes the directory containing the unpacked crx."""
        shutil.rmtree(self._crx_dir)
        self._crx_dir = None


def generate_new_ev_hashes_whitelist(logs_list):
    """Scans the provided logs_list and generates a compressed EV certs
    whitelist for all EV certs in all the logs."""
    all_hashes_set = set()
    for log_url, last_acceptable_entry in logs_list.items():
        output_dir = None
        if FLAGS.output_directory:
            parsed_url = urlparse.urlsplit(log_url)
            output_dir = os.path.join(
                FLAGS.output_directory,
                parsed_url.netloc,
                parsed_url.path.strip("/"))
        print "Scanning log %s, certificates will be in %s" % (
            log_url, output_dir)
        res, hashes_set = (
            generate_ev_whitelist.generate_ev_cert_hashes_from_log(
                log_url,
                FLAGS.multi,
                output_dir,
                last_acceptable_entry))
        print "Scanned %d, %d matched, %d failed parsing (strict/partial)" % (
            res.total, res.matches, res.errors)
        print "There are %d EV hashes." % (len(hashes_set))
        old_hashes_count = len(all_hashes_set)
        all_hashes_set = all_hashes_set.union(hashes_set)
        print "%d new EV certificates." % (
            (len(all_hashes_set) - old_hashes_count))

    hashes_list = list(all_hashes_set)
    hashes_list.sort()
    with open(FLAGS.output_truncated_hashes, "wb") as hashes_file:
        for trimmed_hash in hashes_list:
            hashes_file.write(trimmed_hash)
    golomb_coded_bytes = golomb_code.golomb_encode(
        hashes_list, FLAGS.hash_length, 2 ** FLAGS.two_power)
    return golomb_coded_bytes


def main():
    crx = EVCRXHandler()
    print "Current manifest version is %d." % (crx.manifest_version())
    compressed_hashes = generate_new_ev_hashes_whitelist(LOGS_LIST)
    crx.update_ev_whitelist(compressed_hashes)
    output_file = crx.pack_to(FLAGS.output_crx)
    print "New EV whitelist is at %s" % (output_file)
    crx.cleanup()

if __name__ == "__main__":
    sys.argv = FLAGS(sys.argv)
    main()



#!/usr/bin/env python
"""verify_util.py: CT signature verification utility.

Usage:

  verify_util.py <command> [flags] [cert_file]

Known commands:

  verify_sct: Verify Signed Certificate Timestamp over X.509 certificate.

  The cert_file must contain one or more PEM-encoded certificates.

  For example:

  verify_util.py verify_sct --sct=cert_sct.tls --log_key=log_key.pem cert.pem
"""

import sys
from ct.crypto import cert
from ct.crypto import pem
from ct.crypto import verify
from ct.proto import client_pb2
from ct.serialization import tls_message
import gflags

FLAGS = gflags.FLAGS
gflags.DEFINE_string("sct", None, "TLS-encoded SCT file")
gflags.DEFINE_string("log_key", None, "PEM-encoded CT log key")

def exit_with_message(error_message):
    print error_message
    print "Use --helpshort or --help to get help."
    sys.exit(1)


def verify_sct(chain, sct_tls, log_key_pem):
    sct = client_pb2.SignedCertificateTimestamp()
    tls_message.decode(sct_tls, sct)

    log_key = pem.from_pem(log_key_pem, 'PUBLIC KEY')[0]
    key_info = verify.create_key_info_from_raw_key(log_key)

    lv = verify.LogVerifier(key_info)
    print lv.verify_sct(sct, chain)


def main(argv):
    if len(argv) <= 1 or argv[1][0] == "-":
        # No command. Parse flags anyway to trigger help flags.
        try:
            argv = FLAGS(argv)
            exit_with_message("No command")
        except gflags.FlagsError as e:
            exit_with_message("Error parsing flags: %s" % e)

    argv = argv[1:]

    try:
        argv = FLAGS(argv)
    except gflags.FlagsError as e:
        exit_with_message("Error parsing flags: %s" % e)

    command, cert_file = argv[0:2]

    if command != "verify_sct":
        exit_with_message("Unknown command %s" % command)

    if not cert_file:
        exit_with_message("No certificate file given")

    chain = list(cert.certs_from_pem_file(cert_file, strict_der = False))

    verify_sct(chain,
               open(FLAGS.sct, 'rb').read(),
               open(FLAGS.log_key, 'rb').read())
    sys.exit(0)

if __name__ == "__main__":
    main(sys.argv)



#!/usr/bin/env python
"""Generates a list of hashes of EV certificates found in a log."""

import hashlib
import functools
import pickle
import os
import sys

import ev_metadata
import gflags

from ct.client import scanner
from ct.crypto import cert
from ct.proto import client_pb2

FLAGS = gflags.FLAGS

gflags.DEFINE_integer("hash_trim", 8, "Number of bytes of the SHA-256 digest "
                      "to use in the whitelist.")

gflags.DEFINE_integer("multi", 2, "Number of cert fetching and parsing "
                      "processes to use, in addition to the main process.")

gflags.DEFINE_string("output_directory", None,
                     "Output directory for individual EV certificates. "
                     "If provided, individual EV certs will be written there.")


def calculate_certificate_hash(certificate):
    """Hash the input's DER representation and trim it."""
    hasher = hashlib.sha256(certificate.to_der())
    return hasher.digest()[0:FLAGS.hash_trim]


def find_matching_policies(certificate):
    """Returns the certificate's EV policy OID, if exists."""
    try:
        matching_policies = []
        for policy in certificate.policies():
            if policy['policyIdentifier'] in ev_metadata.EV_POLICIES:
                matching_policies.append(policy['policyIdentifier'])
        return matching_policies
    except cert.CertificateError:
        pass
    return []


def does_root_match_policy(policy_oid, cert_chain):
    """Returns true if the fingerprint of the root certificate matches the
    expected fingerprint for this EV policy OID."""
    if not cert_chain: # Empty chain
        return False
    root_fingerprint = hashlib.sha1(cert_chain[-1]).digest()
    return root_fingerprint in ev_metadata.EV_POLICIES[policy_oid]


def _write_cert_and_chain(
        output_dir, certificate, extra_data, certificate_index):
    """Writes the certificate and its chain to files for later analysis."""
    open(
        os.path.join(output_dir,
                     "cert_%d.der" % certificate_index), "wb"
        ).write(certificate.to_der())

    pickle.dump(
        list(extra_data.certificate_chain),
        open(os.path.join(output_dir,
                          "cert_%d_extra_data.pickle" % certificate_index),
             "wb"))

def _ev_match(
        output_dir, last_acceptable_entry_index, certificate, entry_type,
        extra_data, certificate_index):
    """Matcher function for the scanner. Returns the certificate's hash if
    it is a valid, non-expired, EV certificate, None otherwise."""
    # Only generate whitelist for non-precertificates. It is expected that if
    # a precertificate was submitted then the issued SCT would be embedded
    # in the final certificate.
    if entry_type != client_pb2.X509_ENTRY:
        return None
    # No point in including expired certificates.
    if certificate.is_expired():
        return None
    # Do not include entries beyond the last entry included in the whitelist
    # generated on January 1st, 2015.
    if certificate_index > last_acceptable_entry_index:
        return None

    # Only include certificates that have an EV OID.
    matching_policies = find_matching_policies(certificate)
    if not matching_policies:
        return None

    # Removed the requirement that the root of the chain matches the root that
    # should be used for the EV policy OID.
    # See https://code.google.com/p/chromium/issues/detail?id=524635 for
    # details.

    # Matching certificate
    if output_dir:
        _write_cert_and_chain(
            output_dir, certificate, extra_data, certificate_index)

    return calculate_certificate_hash(certificate)


def generate_ev_cert_hashes_from_log(
        log_url, num_processes, output_directory, last_acceptable_entry_index):
    """Scans the given log and generates a list of hashes for all EV
    certificates in it.

    Returns a tuple of (scan_results, hashes_list)"""
    if output_directory and not os.path.exists(output_directory):
        os.makedirs(output_directory)

    ev_hashes = set()
    def add_hash(cert_hash):
        """Store the hash. Always called from the main process, so safe."""
        ev_hashes.add(cert_hash)
    bound_ev_match = functools.partial(_ev_match, output_directory,
                                       last_acceptable_entry_index)
    res = scanner.scan_log(bound_ev_match, log_url, num_processes, add_hash)
    return (res, ev_hashes)

def main(output_file):
    """Scan and save results to a file."""
    res, hashes_set = generate_ev_cert_hashes_from_log(
        "https://ct.googleapis.com/pilot",
        FLAGS.multi,
        FLAGS.output_directory)
    print "Scanned %d, %d matched and %d failed strict or partial parsing" % (
        res.total, res.matches, res.errors)
    print "There are %d EV hashes." % (len(hashes_set))
    with open(output_file, "wb") as hashes_file:
        hashes_list = list(hashes_set)
        hashes_list.sort()
        for trimmed_hash in hashes_list:
            hashes_file.write(trimmed_hash)


if __name__ == "__main__":
    sys.argv = FLAGS(sys.argv)
    if len(sys.argv) < 2:
        sys.stderr.write(
            "Usage: %s <output file>\n  <output file> will contain the "
            "sorted, truncated hashes list.\n" % sys.argv[0])
        sys.exit(1)
    main(sys.argv[1])



"""Verify CT log statements."""

import io
import struct

from ct.crypto import error
from ct.crypto import pem
from ct.crypto import merkle
from ct.crypto import verify_ecdsa
from ct.crypto import verify_rsa
from ct.crypto.asn1 import oid
from ct.crypto.asn1 import x509_common
from ct.crypto.asn1 import x509_extension as x509_ext
from ct.crypto.asn1 import x509_name
from ct.proto import client_pb2
from ct.proto import ct_pb2
from ct.serialization import tls_message

SUPPORTED_SIGNATURE_ALGORITHMS = (
    ct_pb2.DigitallySigned.ECDSA,
    ct_pb2.DigitallySigned.RSA
)

def decode_signature(signature):
    """Decode the TLS-encoded serialized signature.

    Args:
        signature: TLS-encoded signature.

    Returns:
        a tuple of (hash algorithm, signature algorithm, signature data)

    Raises:
        ct.crypto.error.EncodingError: invalid TLS encoding.
    """

    sig_stream = io.BytesIO(signature)

    sig_prefix = sig_stream.read(2)
    if len(sig_prefix) != 2:
        raise error.EncodingError("Invalid algorithm prefix %s" %
                                      sig_prefix.encode("hex"))
    hash_algo, sig_algo = struct.unpack(">BB", sig_prefix)
    if (hash_algo != ct_pb2.DigitallySigned.SHA256 or
        sig_algo not in SUPPORTED_SIGNATURE_ALGORITHMS):
        raise error.EncodingError("Invalid algorithm(s) %d, %d" %
                                  (hash_algo, sig_algo))

    length_prefix = sig_stream.read(2)
    if len(length_prefix) != 2:
        raise error.EncodingError("Invalid signature length prefix %s" %
                                  length_prefix.encode("hex"))
    sig_length, = struct.unpack(">H", length_prefix)
    remaining = sig_stream.read()
    if len(remaining) != sig_length:
        raise error.EncodingError("Invalid signature length %d for "
                                  "signature %s with length %d" %
                                  (sig_length, remaining.encode("hex"),
                                   len(remaining)))
    return (hash_algo, sig_algo, remaining)

def _get_precertificate_issuer(chain):
    try:
        issuer = chain[1]
    except IndexError:
        raise error.IncompleteChainError(
                "Chain with PreCertificate must contain issuer.")

    if not issuer.extended_key_usage(oid.CT_PRECERTIFICATE_SIGNING):
        return issuer
    else:
        try:
            return chain[2]
        except IndexError:
            raise error.IncompleteChainError(
                "Chain with PreCertificate signed by PreCertificate "
                "Signing Cert must contain issuer.")

def _find_extension(asn1, extn_id):
    """Find an extension from a certificate's ASN.1 representation

    Args:
        asn1: x509.Certificate instance.
        extn_id: OID of the extension to look for.

    Returns:
        The decoded value of the extension, or None if not found.
        This is a reference and can be modified.
    """
    for e in asn1["tbsCertificate"]["extensions"]:
        if e["extnID"] == extn_id:
            return e["extnValue"].decoded_value

    return None

def _remove_extension(asn1, extn_id):
    """Remove an extension from a certificate's ASN.1 representation

    Args:
        asn1: x509.Certificate instance.
        extn_id: OID of the extension to be removed.
    """
    asn1["tbsCertificate"]["extensions"] = (
        filter(lambda e: e["extnID"] != extn_id,
               asn1["tbsCertificate"]["extensions"])
    )

def _encode_tbs_certificate_for_validation(cert, issuer):
    """Normalize a Certificate for CT Signing / Verification
    The poison and embedded sct extensions are removed if present,
    and the issuer information is changed to match the one given in
    argument. The resulting TBS certificate is encoded.

    Args:
        cert: Certificate instance to be normalized
        issuer: Issuer certificate used to fix issuer information in TBS
                certificate.

    Returns:
        DER encoding of the normalized TBS Certificate
    """
    asn1 = cert.to_asn1()
    issuer_asn1 = issuer.to_asn1()

    _remove_extension(asn1, oid.CT_POISON)
    _remove_extension(asn1, oid.CT_EMBEDDED_SCT_LIST)
    asn1["tbsCertificate"]["issuer"] = issuer_asn1["tbsCertificate"]["subject"]

    akid = _find_extension(asn1, oid.ID_CE_AUTHORITY_KEY_IDENTIFIER)
    if akid is not None:
        akid[x509_ext.KEY_IDENTIFIER] = issuer.subject_key_identifier()
        akid[x509_ext.AUTHORITY_CERT_SERIAL_NUMBER] = issuer.serial_number()
        akid[x509_ext.AUTHORITY_CERT_ISSUER] = [
            x509_name.GeneralName({
                x509_name.DIRECTORY_NAME: issuer_asn1["tbsCertificate"]["issuer"]
            })
        ]

    return asn1["tbsCertificate"].encode()

def _is_precertificate(cert):
    return (cert.has_extension(oid.CT_POISON) or
            cert.has_extension(oid.CT_EMBEDDED_SCT_LIST))

def _create_dst_entry(sct, chain):
    """Create a Digitally Signed Timestamped Entry to be validated

    Args:
        sct: client_pb2.SignedCertificateTimestamp instance.
        chain: list of Certificate instances.

    Returns:
        client_pb2.DigitallySignedTimestampedEntry instance with all
        fields set.

    Raises:
        ct.crypto.error.IncompleteChainError: a certificate is missing
            from the chain.
    """

    try:
        leaf_cert = chain[0]
    except IndexError:
        raise error.IncompleteChainError(
                "Chain must contain leaf certificate.")

    entry = client_pb2.DigitallySignedTimestampedEntry()
    entry.sct_version = ct_pb2.V1
    entry.signature_type = client_pb2.CERTIFICATE_TIMESTAMP
    entry.timestamp = sct.timestamp
    entry.ct_extensions = sct.extensions

    if _is_precertificate(leaf_cert):
        issuer = _get_precertificate_issuer(chain)

        entry.entry_type = client_pb2.PRECERT_ENTRY
        entry.pre_cert.issuer_key_hash = issuer.key_hash('sha256')
        entry.pre_cert.tbs_certificate = (
            _encode_tbs_certificate_for_validation(leaf_cert, issuer)
        )
    else:
        entry.entry_type = client_pb2.X509_ENTRY
        entry.asn1_cert = leaf_cert.to_der()

    return entry


class LogVerifier(object):
    """CT log verifier."""

    def __init__(self, key_info, merkle_verifier=merkle.MerkleVerifier()):
        """Initialize from KeyInfo protocol buffer and a MerkleVerifier."""
        self.__merkle_verifier = merkle_verifier
        if (key_info.type == client_pb2.KeyInfo.ECDSA):
            self.__sig_verifier = verify_ecdsa.EcdsaVerifier(key_info)
        elif (key_info.type == client_pb2.KeyInfo.RSA):
            self.__sig_verifier = verify_rsa.RsaVerifier(key_info)
        else:
            raise error.UnsupportedAlgorithmError("Key type %d not supported" %
                                                  key_info.type)

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self.__sig_verifier)

    def _encode_sth_input(self, sth_response):
        if len(sth_response.sha256_root_hash) != 32:
            raise error.EncodingError("Wrong hash length: expected 32, got %d" %
                                      len(sth_response.sha256_root_hash))
        return struct.pack(">BBQQ32s", ct_pb2.V1, ct_pb2.TREE_HEAD,
                           sth_response.timestamp, sth_response.tree_size,
                           sth_response.sha256_root_hash)

    @error.returns_true_or_raises
    def _assert_correct_signature_algorithms(self, hash_algo, sig_algo):
        if (hash_algo != self.__sig_verifier.HASH_ALGORITHM):
            raise error.SignatureError(
                "Hash algorithm used for the signature (%d) does not match the "
                "one used for the public key (%d)" %
                (hash_algo, self.__sig_verifier.HASH_ALGORITHM))

        if (sig_algo != self.__sig_verifier.SIGNATURE_ALGORITHM):
            raise error.SignatureError(
                "Signing algorithm used (%d) does not match the one used for "
                "the public key (%d)" %
                (sig_algo, self.__sig_verifier.SIGNATURE_ALGORITHM))

        return True

    @error.returns_true_or_raises
    def verify_sth(self, sth_response):
        """Verify the STH Response.

        Args:
            sth_response: client_pb2.SthResponse proto. The response must have
                all fields present.

        Returns:
            True. The return value is enforced by a decorator and need not be
                checked by the caller.

        Raises:
            ct.crypto.error.EncodingError: failed to encode signature input,
                or decode the signature.
            ct.crypto.error.SignatureError: invalid signature.
        """
        signature_input = self._encode_sth_input(sth_response)

        (hash_algo, sig_algo, signature) = decode_signature(
            sth_response.tree_head_signature)

        self._assert_correct_signature_algorithms(hash_algo, sig_algo)

        return self.__sig_verifier.verify(signature_input, signature)

    @staticmethod
    @error.returns_true_or_raises
    def verify_sth_temporal_consistency(old_sth, new_sth):
        """Verify the temporal consistency for two STH responses.

        For two STHs, verify that the newer STH has bigger tree size.
        Does not verify STH signatures or consistency of hashes.

        Args:
            old_sth: client_pb2.SthResponse proto. The STH with the older
                timestamp must be supplied first.
            new_sth: client_pb2.SthResponse proto.

        Returns:
            True. The return value is enforced by a decorator and need not be
                checked by the caller.

        Raises:
            ct.crypto.error.ConsistencyError: STHs are inconsistent
            ValueError: "Older" STH is not older.
        """
        if old_sth.timestamp > new_sth.timestamp:
            raise ValueError("Older STH has newer timestamp (%d vs %d), did "
                             "you supply inputs in the wrong order?" %
                             (old_sth.timestamp, new_sth.timestamp))

        if (old_sth.timestamp == new_sth.timestamp and
            old_sth.tree_size != new_sth.tree_size):
            # Issuing two different STHs for the same timestamp is illegal,
            # even if they are otherwise consistent.
            raise error.ConsistencyError("Inconsistency: different tree sizes "
                                         "for the same timestamp")
        if (old_sth.timestamp < new_sth.timestamp and
            old_sth.tree_size > new_sth.tree_size):
            raise error.ConsistencyError("Inconsistency: older tree has bigger "
                                         "size")
        return True

    @error.returns_true_or_raises
    def verify_sth_consistency(self, old_sth, new_sth, proof):
        """Verify consistency of two STHs.

        Verify the temporal consistency and consistency proof for two STH
        responses. Does not verify STH signatures.

        Args:
            old_sth: client_pb2.SthResponse() proto. The STH with the older
                timestamp must be supplied first.
            new_sth: client_pb2.SthResponse() proto.
            proof: a list of SHA256 audit nodes.

        Returns:
            True. The return value is enforced by a decorator and need not be
                checked by the caller.

        Raises:
            ConsistencyError: STHs are inconsistent
            ProofError: proof is invalid
            ValueError: "Older" STH is not older.
        """
        self.verify_sth_temporal_consistency(old_sth, new_sth)
        self.__merkle_verifier.verify_tree_consistency(
            old_sth.tree_size, new_sth.tree_size, old_sth.sha256_root_hash,
            new_sth.sha256_root_hash, proof)
        return True

    @error.returns_true_or_raises
    def verify_sct(self, sct, chain):
        """Verify the SCT over the X.509 certificate provided

        Args:
            sct: client_pb2.SignedCertificateTimestamp proto. Must have
                all fields present.
            chain: list of cert.Certificate instances. Begins with the
                certificate to be checked.

        Returns:
            True. The return value is enforced by a decorator and need not be
                checked by the caller.

        Raises:
            ct.crypto.error.EncodingError: failed to encode signature input,
                or decode the signature.
            ct.crypto.error.SignatureError: invalid signature.
            ct.crypto.error.IncompleteChainError: a certificate is missing
                from the chain.
        """

        if sct.version != ct_pb2.V1:
            raise error.UnsupportedVersionError("Cannot handle version: %s" %
                                                sct.version)
        entry = _create_dst_entry(sct, chain)
        signature_input = tls_message.encode(entry)

        self._assert_correct_signature_algorithms(sct.signature.hash_algorithm,
                                                  sct.signature.sig_algorithm)

        return self.__sig_verifier.verify(signature_input,
                                          sct.signature.signature)

    def verify_embedded_scts(self, chain):
        """Extract and verify SCTs embedded in an X.509 certificate.

        Args:
            chain: list of cert.Certificate instances.

        Returns:
            List of (SignedCertificateTimestamp, bool) pairs, one for each SCT
                present in the certificate. The boolean is True if the
                corresponding SCT is valid, False otherwise.

        Raises:
            ct.crypto.error.EncodingError: failed to encode signature input,
                or decode the signature.
            ct.crypto.error.IncompleteChainError: the chain is empty.
        """

        try:
            leaf_cert = chain[0]
        except IndexError:
            raise error.IncompleteChainError(
                    "Chain must contain leaf certificate.")

        scts_blob = leaf_cert.embedded_sct_list()
        if scts_blob is None:
            return []

        scts = client_pb2.SignedCertificateTimestampList()
        tls_message.decode(scts_blob, scts)

        result = []
        for sct_blob in scts.sct_list:
            sct = client_pb2.SignedCertificateTimestamp()
            tls_message.decode(sct_blob, sct)

            try:
                self.verify_sct(sct, chain)
                result.append((sct, True))
            except error.VerifyError:
                result.append((sct, False))

        return result


def create_key_info_from_raw_key(log_key):
    """Creates a KeyInfo from the given raw (DER-encoded) key.

    Detects the key type (ECDSA or RSA), returning a client_pb2.KeyInfo
    instance that can be used to construct a LogVerifier.

    Args:
        log_key: A DER-encoded key.

    Returns:
        A client_pb2.KeyInfo instance with all fields correctly filled.
    """
    key_info = client_pb2.KeyInfo()
    decoded_key = x509_common.SubjectPublicKeyInfo.decode(log_key)
    key_algorithm_oid = decoded_key['algorithm']['algorithm']
    if key_algorithm_oid == oid.RSA_ENCRYPTION:
        key_info.type = client_pb2.KeyInfo.RSA
    elif key_algorithm_oid == oid.ID_EC_PUBLICKEY:
        key_info.type = client_pb2.KeyInfo.ECDSA
    else:
        raise error.UnsupportedAlgorithmError(
                'Unknown key type: %s' % key_algorithm_oid)
    key_info.pem_key = pem.to_pem(log_key, 'PUBLIC KEY')
    return key_info



from ct.client import reporter

class AggregatedCertificateReport(reporter.CertificateReport):
    """Reporter which passes the function calls to other reporters passed through
    constructor."""
    def __init__(self, reporters):
        self._reporters = reporters
        super(AggregatedCertificateReport, self).__init__()

    def report(self):
        super(AggregatedCertificateReport, self).report()
        for report in self._reporters:
            report.report()

    def reset(self):
        for report in self._reporters:
            report.reset()

    def _batch_scanned_callback(self, result):
        for report in self._reporters:
            report._batch_scanned_callback(result)



import base64
import dns.resolver
import dns.rdatatype
import logging
import math
import os
import random
import shlex
import signal
import subprocess
import sys
import time

NUMBER_OF_CERTS = 100

basepath = os.path.dirname(sys.argv[0])

sys.path.append(os.path.join(basepath, '../../python'))
from ct.crypto import merkle
from ct.proto import ct_pb2

tmpdir = sys.argv[1]

class CTDNSLookup:
    def __init__(self, nameservers, port):
        self.resolver = dns.resolver.Resolver(configure=False)
        self.resolver.nameservers = nameservers
        self.resolver.port = port

    def Get(self, name):
        answers = self.resolver.query(name, 'TXT')
        assert answers.rdtype == dns.rdatatype.TXT
        return answers

    def GetOne(self, name):
        answers = self.Get(name)
        assert len(answers) == 1
        txt = answers[0]
        assert len(txt.strings) == 1
        return txt.strings[0]

    def GetSTH(self):
        sth_str = self.GetOne('sth.example.com')
        sth = ct_pb2.SignedTreeHead()
        parts = str(sth_str).split('.')
        sth.tree_size = int(parts[0])
        sth.timestamp = int(parts[1])
        sth.sha256_root_hash = base64.b64decode(parts[2])
        #FIXME(benl): decompose signature into its parts
        #sth.signature = base64.b64decode(parts[3])
        return sth

    def GetEntry(self, level, index, size):
        return self.GetOne(str(level) + '.' + str(index) + '.' + str(size)
                           + '.tree.example.com')

    def GetLeafHash(self, index):
        return self.GetOne(str(index) + '.leafhash.example.com')

class DNSServerRunner:
    def Run(self, cmd):
        args = shlex.split(cmd)
        self.proc = subprocess.Popen(args)

def OpenSSL(*params):
    logging.info("RUN: openssl " + str(params))
    null = open("/dev/null")
    subprocess.check_call(("openssl",) + params, stdout=null, stderr=null)

class timeout:
    def __init__(self, seconds, error_message='Timeout'):
        self.seconds = seconds
        self.error_message = error_message

    def handle_timeout(self, signum, frame):
        raise TimeoutError(self.error_message)

    def __enter__(self):
        signal.signal(signal.SIGALRM, self.handle_timeout)
        signal.alarm(self.seconds)

    def __exit__(self, type, value, traceback):
        signal.alarm(0)

class CTServer:
    def __init__(self, cmd, base, ca):
        self.cmd_ = cmd
        self.base_ = base
        self.ca_ = ca
        self.GenerateKey()

    def __del__(self):
        self.proc.terminate()
        self.proc.wait()

    def PrivateKey(self):
        return self.base_ + "-ct-server-private-key.pem"
        
    def PublicKey(self):
        return self.base_ + "-ct-server-public-key.pem"

    def Database(self):
        return self.base_ + "-database.sqlite"

    def GenerateKey(self):
        OpenSSL("ecparam",
                "-out", self.PrivateKey(),
                "-name", "secp256r1",
                "-genkey")
        OpenSSL("ec",
                "-in", self.PrivateKey(),
                "-pubout",
                "-out", self.PublicKey())

    def URL(self):
        return "http://localhost:9999/"

    def Run(self):
        cmd = (self.cmd_ + " -key " + self.PrivateKey() +
               " -trusted_cert_file " + self.ca_.RootCertificate() +
               " -sqlite_db " + self.Database() +
               " -tree_signing_frequency_seconds 1" +
               " -logtostderr")
        logging.info("RUN: " + cmd)
        args = shlex.split(cmd)
        self.proc = subprocess.Popen(args, stdout=subprocess.PIPE)
        with timeout(10):
            while self.proc.stdout.readline() != "READY\n":
                continue

RootConfig = """[ req ]
distinguished_name=req_distinguished_name
prompt=no
x509_extensions=v3_ca

[ req_distinguished_name ]
countryName=GB
stateOrProvinceName=Wales
localityName=Erw Wen
0.organizationName=Certificate Transparency Test CA

[ v3_ca ]
subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid:always,issuer:always
basicConstraints=CA:TRUE
"""

CAConfig = """[ ca ]
default_ca = CA_default

[ CA_default ]
default_startdate = 120601000000Z
default_enddate   = 220601000000Z
default_md	  = sha1
unique_subject	  = no
email_in_dn	  = no
policy	          = policy_default
serial            = {serial}
database          = {database}

[ policy_default ]
countryName	    = supplied
organizationName    = supplied
stateOrProvinceName = optional
localityName	    = optional
commonName          = optional
"""

RequestConfig = """[ req ]
distinguished_name=req_distinguished_name
prompt=no

[ req_distinguished_name ]
countryName=GB
stateOrProvinceName=Wales
localityName=Erw Wen
0.organizationName={subject}

# For the precert
[ pre ]
subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid:always,issuer:always
basicConstraints=CA:FALSE
1.3.6.1.4.1.11129.2.4.3=critical,ASN1:NULL

# For the simple cert, without embedded proof extensions
[ simple ]
subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid:always,issuer:always
basicConstraints=CA:FALSE

# For the cert with an embedded proof
[ embedded ]
subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid:always,issuer:always
basicConstraints=CA:FALSE
"""

def WriteFile(name, content):
    with open(name, "w") as f:
        f.write(content)

class CA:
    def __init__(self, base):
        self.base_ = base

        os.mkdir(self.Directory())
        
        open(self.Database(), "w")
        WriteFile(self.Serial(), "0000000000000001")

        WriteFile(self.RootConfig(), RootConfig)
        ca_config = CAConfig.format(database = self.Database(),
                                    serial = self.Serial())
        WriteFile(self.CAConfig(), ca_config)

        self.GenerateRootCertificate()

        os.mkdir(self.IssuedCertificates())

    def Directory(self):
        """Where the CA does house-keeping"""
        return self.base_ + "-housekeeping"

    def Database(self):
        return self.Directory() + "/database"

    def Serial(self):
        return self.Directory() + "/serial"

    def RootConfig(self):
        return self.base_ + "-root-config"

    def CAConfig(self):
        return self.base_ + "-ca-config"

    def PrivateKey(self):
        return self.base_ + "-private-key.pem"

    def RootCertificate(self):
        return self.base_ + "-cert.pem"

    def TempFile(self, name):
        return self.base_ + "-temp-" + name

    def RequestConfig(self):
        return self.TempFile("req-config")

    def IssuedCertificates(self):
        return self.base_ + "-issued"

    def IssuedFile(self, name, subject):
        return self.IssuedCertificates() + "/" + name + "-" + subject + ".pem"

    def IssuedPrivateKey(self, subject):
        return self.IssuedFile("private-key", subject)

    def IssuedCertificate(self, subject):
        return self.IssuedFile("certificate", subject)

    def GenerateRootCertificate(self):
        csr = self.TempFile("csr")
        OpenSSL("req",
                "-new",
                "-newkey", "rsa:2048",
                "-keyout", self.PrivateKey(),
                "-out", csr,
                "-config", self.RootConfig(),
                "-nodes")
        OpenSSL("ca",
                "-in", csr,
                "-selfsign",
                "-keyfile", self.PrivateKey(),
                "-config", self.CAConfig(),
                "-extfile", self.RootConfig(),
                "-extensions", "v3_ca",
                "-outdir", self.Directory(),
                "-out", self.RootCertificate(),
                "-batch")

    def CreateAndLogCert(self, ct_server, subject):
        WriteFile(self.RequestConfig(), RequestConfig.format(subject=subject))
        csr = self.TempFile("csr")
        OpenSSL("req",
                "-new",
                "-newkey", "rsa:1024",
                "-keyout", self.IssuedPrivateKey(subject),
                "-out", csr,
                "-config", self.RequestConfig(),
                "-nodes")
        OpenSSL("ca",
                "-in", csr,
                "-cert", self.RootCertificate(),
                "-keyfile", self.PrivateKey(),
                "-config", self.CAConfig(),
                "-extfile", self.RequestConfig(),
                "-extensions", "simple",
                "-outdir", self.Directory(),
                "-out", self.IssuedCertificate(subject),
                "-batch")

        # Reverse the order of these to show a bug in ct-server where
        # it accepts the CA cert even though there's a redundant extra
        # cert in the chain. At least, I think its a bug.
        certs = (open(self.IssuedCertificate(subject)).read()
                 + open(self.RootCertificate()).read())
        chain_file = self.TempFile("chain")
        WriteFile(chain_file, certs)
        subprocess.check_call(("client/ct", "upload",
                               "-ct_server_submission", chain_file,
                               "-ct_server", ct_server.URL(),
                               "-ct_server_public_key", ct_server.PublicKey(),
                               "-ct_server_response_out", self.TempFile("sct")))

logging.basicConfig(level="WARNING")

# Set up our test CA
ca = CA(tmpdir + "/ct-test-ca")

# Run a CT server
ct_cmd = basepath + "/ct-server"
ct_server = CTServer(ct_cmd, tmpdir + "/ct-test", ca)
ct_server.Run()

# Add nn certs to the CT server
for x in range(NUMBER_OF_CERTS):
    ca.CreateAndLogCert(ct_server, "TestCertificate" + str(x))

# Make sure server has had enough time to assimilate all certs
time.sleep(2)

# We'll need the DB for the DNS server
db = ct_server.Database()
# Kill the CT server (shared database access not currently supported)
del ct_server

# Now run the DNS server from the same database
server_cmd = basepath + "/ct-dns-server --port=1111 --domain=example.com. --db=" + db
runner = DNSServerRunner()
runner.Run(server_cmd)

# Get the STH
lookup = CTDNSLookup(['127.0.0.1'], 1111)

sth = lookup.GetSTH()
logging.info("sth = " + str(sth))
logging.info("size = " + str(sth.tree_size))

assert sth.tree_size == NUMBER_OF_CERTS

# test all the entries
for index in range(NUMBER_OF_CERTS):
    leaf_hash = lookup.GetLeafHash(index)
    logging.info("index = " + str(index) + " hash = " + leaf_hash)

    verifier = merkle.MerkleVerifier()
    audit_path = []
    for level in range(0, verifier.audit_path_length(index, sth.tree_size)):
        hash = lookup.GetEntry(level, index, sth.tree_size)
        logging.info("hash = " + hash)
        audit_path.append(base64.b64decode(hash))

    logging.info("path = " + str(map(base64.b64encode, audit_path)))

    assert verifier.verify_leaf_hash_inclusion(base64.b64decode(leaf_hash),
                                               index, audit_path, sth)

print "DNS Server test passed"



import cPickle as pickle

from verifiable_log import VerifiableLog
from verifiable_map import VerifiableMap

# Extend this class, override _apply_operation and add your own API to:
# 1. append to log
# 2. read from map
class VerifiableBase:
  def __init__(self, log):
    # The log, such as a VerifiableLog
    self._log = log

    # Internal map that we use.  The mapper is responsible for mutating this
    # when triggered by log changes.
    self._map = VerifiableMap()

    # How many log changes have been processed
    self._ops_processed = 0

    # After we process a log operation, we capture the corresponding map
    # mutation index which may be higher or lower.
    self._log_sth_to_map_sth = {0: 0}

  # Called internally to poll the log and process all updates
  def _update_from_log(self):
    log_size = self._log.get_tree_head()['tree_size']
    ctr = 0
    while log_size > self._ops_processed:
      for entry in self._log.get_entries(self._ops_processed, log_size - 1):
        # Call mapper
        self._apply_operation(self._ops_processed, entry, self._map)
        self._ops_processed += 1
        self._log_sth_to_map_sth[self._ops_processed] = self._map.get_tree_head()['tree_size']

  # Called by the underlying map when new entries are sequenced by the log
  # Override me!
  def _apply_operation(self, idx, entry, map):
    pass

  # Get the value and proof for a key.  Tree size the number of entries in the log
  def get(self, key, tree_size):
    if tree_size > self._ops_processed:
      raise ValueError
    return self._map.get(key, self._log_sth_to_map_sth[tree_size])

  # Return the current tree head, this triggers fetching the latest entries
  # from the log (if needed) and this tree_size should be passed to corresponding
  # get() calls.
  def get_tree_head(self, tree_size=None):
    if tree_size is None or tree_size > self._ops_processed:
      self._update_from_log()
      if tree_size is None:
        tree_size = self._ops_processed
    if tree_size > self._ops_processed:
      raise ValueError
    rv = self._map.get_tree_head(self._log_sth_to_map_sth[tree_size])
    rv['tree_size'] = tree_size # override what the map says
    rv['log_tree_head'] = self._log.get_tree_head(tree_size)
    return rv

  def get_log_entries(self, start, end):
    return self._log.get_entries(start, end)

  def get_log_consistency(self, first, second):
    return self._log.consistency_proof(first, second)

  # Get the value and proof for a key.  Tree size the number of entries in the log
  def debug_dump(self, tree_size):
    return self._map._root.debug_dump(self._log_sth_to_map_sth[self.get_tree_head(tree_size)['tree_size']])



#!/usr/bin/env python

import unittest

import os
import tempfile

from ct.client.db import sqlite_connection as sqlitecon
from ct.client.db import sqlite_temp_db
from ct.client.db import temp_db_test

class SQLiteTempDBTest(unittest.TestCase, temp_db_test.TempDBTest):
    def setUp(self):
        self.database = sqlite_temp_db.SQLiteTempDB(
            sqlitecon.SQLiteConnectionManager(":memory:", keepalive=True))
    def db(self):
        return self.database

class SQLiteTempDBFactoryTest(unittest.TestCase):
    def setUp(self):
        self.database_dir = tempfile.mkdtemp()
        self.factory = sqlite_temp_db.SQLiteTempDBFactory(
            sqlitecon.SQLiteConnectionManager(":memory:", keepalive=True),
            self.database_dir)
        self.temp_stores = set()

    def create_storage(self, name):
        storage = self.factory.create_storage(name)
        self.temp_stores.add(storage.db_name())
        return storage

    def tearDown(self):
        for temp_store in self.temp_stores:
            os.remove(temp_store)
        os.rmdir(self.database_dir)

    def test_create_storage(self):
        temp = self.create_storage("log_server")
        self.assertEqual(self.database_dir, os.path.dirname(temp.db_name()))

    def test_create_storage_creates_unique(self):
        temp1 = self.create_storage("log_server1")
        temp2 = self.create_storage("log_server2")
        self.assertNotEqual(temp1.db_name(), temp2.db_name())

    def test_create_storage_remembers_mapping(self):
        temp = self.create_storage("log_server")
        temp2 = self.create_storage("log_server")
        self.assertEqual(temp.db_name(), temp2.db_name())

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import unittest

import os
import tempfile

from ct.client import state
from ct.proto import client_pb2

class StateKeeperTest(unittest.TestCase):
    _DEFAULT_STATE = client_pb2.MonitorState()
    # fill with some data
    _DEFAULT_STATE.verified_sth.timestamp = 1234
    _DEFAULT_STATE.pending_sth.tree_size = 5678

    def test_read_write(self):
        handle, state_file = tempfile.mkstemp()
        os.close(handle)
        state_keeper = state.StateKeeper(state_file)
        state_keeper.write(self._DEFAULT_STATE)
        self.assertEqual(self._DEFAULT_STATE,
                         state_keeper.read(client_pb2.MonitorState))
        os.remove(state_file)

    def test_read_no_such_file(self):
        temp_dir = tempfile.mkdtemp()
        state_keeper = state.StateKeeper(temp_dir + "/foo")
        self.assertRaises(state.FileNotFoundError, state_keeper.read,
                          client_pb2.MonitorState)

    def test_read_corrupt_file(self):
        handle, state_file = tempfile.mkstemp()
        os.write(handle, "wibble")
        os.close(handle)
        state_keeper = state.StateKeeper(state_file)
        self.assertRaises(state.CorruptStateError,
                          state_keeper.read, client_pb2.MonitorState)
        os.remove(state_file)

if __name__ == "__main__":
    unittest.main()



import logging
import threading
import gflags
from ct.client import reporter
from Queue import Queue

FLAGS = gflags.FLAGS

gflags.DEFINE_integer("cert_db_writer_queue_size", 10, "Size of certificate "
                      "queue in db reporter")


class CertDBCertificateReport(reporter.CertificateReport):
    def __init__(self, cert_db, log_key):
        self._cert_db = cert_db
        self.log_key = log_key
        self._certs_queue = Queue(FLAGS.cert_db_writer_queue_size)
        self._writer = None
        super(CertDBCertificateReport, self).__init__()

    def _writer_ready(self):
        return self._writer and self._writer.is_alive()

    def report(self):
        super(CertDBCertificateReport, self).report()

        if self._writer_ready():
            self._certs_queue.join()
            logging.info("Finished scheduled writing to CertDB")
            self._certs_queue.put(None)
            self.reset()

    def reset(self):
        if self._writer_ready():
            self._writer.join()
            self._writer = None

    def _batch_scanned_callback(self, result):
        if not self._writer_ready():
            self._writer = threading.Thread(target=_process_certs,
                                            args=(self._cert_db, self.log_key,
                                                  self._certs_queue))
            self._writer.start()
        self._certs_queue.put([(desc, index) for desc, index in result])


def _process_certs(db, log_key, certs_queue):
    while True:
        certs = certs_queue.get()
        try:
            # This check must be performed in the try block so task_done will
            # be invoked in the finally block regardless of the check results.
            if not certs:
                break
            db.store_certs_desc(certs, log_key)
        except:
            logging.exception("Failed to store certificate information")
        finally:
            certs_queue.task_done()



#!/usr/bin/env python

# Get the issuers for the supplied certificates from the AIA
# extension. Note that this is not very clever and may supply
# intermediates that are not needed and fail to supply intermediates
# that are needed.

# Based on pyasn1 example code.

from base64 import b64encode
from pyasn1.codec.der import decoder
from pyasn1.codec.der import encoder
from pyasn1.type import namedtype
from pyasn1.type import univ
from pyasn1_modules import pem
from pyasn1_modules import rfc2459
import sys
from urllib2 import urlopen

if len(sys.argv) != 1:
  print """Usage:
  $ %s < somecertificates.pem""" % sys.argv[0]
  sys.exit(-1)

cStart = '-----BEGIN CERTIFICATE-----'
cEnd = '-----END CERTIFICATE-----'

certType = rfc2459.Certificate()

# RFC 2459 is not sufficient for X509v3 certificates, extra stuff here.
# RFC 5280 4.2.2.1

id_pe_authorityInfoAccess = univ.ObjectIdentifier('1.3.6.1.5.5.7.1.1')

class AccessDescription(univ.Sequence):
  """
     AccessDescription  ::=  SEQUENCE {
                accessMethod          OBJECT IDENTIFIER,
                accessLocation        GeneralName  }
  """
  componentType = namedtype.NamedTypes(
    namedtype.NamedType('accessMethod', univ.ObjectIdentifier()),
    namedtype.NamedType('accessLocation', rfc2459.GeneralName()))

class AuthorityInfoAccessSyntax(univ.SequenceOf):
  """
  AuthorityInfoAccessSyntax  ::=
             SEQUENCE SIZE (1..MAX) OF AccessDescription
  """
  # FIXME: SIZE not encoded.
  componentType = AccessDescription()

id_ad_caIssuers = univ.ObjectIdentifier('1.3.6.1.5.5.7.48.2')

# End of RFC 5280 4.2.2.1

certCnt = 0

while 1:
  idx, substrate = pem.readPemBlocksFromFile(
    sys.stdin, (cStart, cEnd)
    )
  if not substrate:
    break

  cert, rest = decoder.decode(substrate, asn1Spec=certType)

  if rest: substrate = substrate[:-len(rest)]

  print cert.prettyPrint()

  tbs = cert.getComponentByName('tbsCertificate')
  extensions = tbs.getComponentByName('extensions') or []

  for extension in extensions:
    oid = extension.getComponentByName('extnID')
    if oid != id_pe_authorityInfoAccess:
      continue
    
    print extension.prettyPrint()

    value, rest = decoder.decode(extension.getComponentByName('extnValue'),
                                 asn1Spec=univ.OctetString())
    assert rest == ""
    aia, rest = decoder.decode(value, asn1Spec=AuthorityInfoAccessSyntax())
    assert rest == ""

    print aia.prettyPrint()

    for ad in aia:
      oid = ad.getComponentByName('accessMethod')
      if oid != id_ad_caIssuers:
        continue
      
      print ad.prettyPrint()

      loc = ad.getComponentByName('accessLocation').\
        getComponentByName('uniformResourceIdentifier')
      print type(loc), loc

      certHandle = urlopen(str(loc))
      cert = certHandle.read()
      print cStart
      b64 = b64encode(cert)
      for n in range(0, len(b64), 64):
        print b64[n:n+64]
      print cEnd
    
  certCnt = certCnt + 1

print('*** %s PEM cert(s) de/serialized' % certCnt)



#!/usr/bin/env python
import unittest

import sys
from collections import defaultdict
from ct.client import reporter
from ct.client.db import cert_desc
from ct.client.db import sqlite_cert_db
from ct.client.db import sqlite_connection as sqlitecon
from ct.crypto import cert
from ct.proto import certificate_pb2
from ct.proto import client_pb2
from ct.test import test_config
import gflags

STRICT_DER = cert.Certificate.from_der_file(
        test_config.get_test_file_path('google_cert.der'), False).to_der()
NON_STRICT_DER = cert.Certificate.from_pem_file(
        test_config.get_test_file_path('invalid_ip.pem'), False).to_der()

CHAIN_FILE = test_config.get_test_file_path('google_chain.pem')

CHAIN_DERS = [c.to_der() for c in cert.certs_from_pem_file(CHAIN_FILE)]

SELF_SIGNED_ROOT_DER = cert.Certificate.from_pem_file(
        test_config.get_test_file_path('subrigo_net.pem'), False).to_der()

def readable_dn(dn_attribs):
    return ",".join(["%s=%s" % (attr.type, attr.value) for attr in dn_attribs])

class CertificateReportTest(unittest.TestCase):
    class CertificateReportBase(reporter.CertificateReport):
        def __init__(self):
            super(CertificateReportTest.CertificateReportBase, self).__init__()

        def report(self):
            super(CertificateReportTest.CertificateReportBase, self).report()
            return self._certs

        def reset(self):
            self._certs = {}

        def _batch_scanned_callback(self, result):
            for desc, log_index in result:
                self._certs[log_index] = desc

    def setUp(self):
        self.cert_db = sqlite_cert_db.SQLiteCertDB(
                sqlitecon.SQLiteConnectionManager(":memory:", keepalive=True))

    def test_scan_der_cert(self):
        report = self.CertificateReportBase()
        report.scan_der_certs([(0, STRICT_DER, [], client_pb2.X509_ENTRY)])
        results = report.report()
        self.assertEqual(len(results), 1)

    def test_scan_der_cert_broken_cert(self):
        report = self.CertificateReportBase()
        report.scan_der_certs([(0, "asdf", [], client_pb2.X509_ENTRY)])
        results = report.report()
        self.assertEqual(len(results), 1)
        self.assertFalse(results[0].subject)

    def test_scan_der_cert_check_non_strict(self):
        report = self.CertificateReportBase()
        report.scan_der_certs([(0, NON_STRICT_DER, [], client_pb2.X509_ENTRY)])
        results = report.report()
        self.assertEqual(len(results), 1)
        self.assertTrue(results[0].subject)

    def test_entry_type_propogated(self):
        report = self.CertificateReportBase()
        report.scan_der_certs([(0, STRICT_DER, [], client_pb2.PRECERT_ENTRY),
                               (1, STRICT_DER, [], client_pb2.X509_ENTRY)])
        results = report.report()
        self.assertEqual(len(results), 2)

        self.assertEquals(results[0].entry_type, client_pb2.PRECERT_ENTRY)
        self.assertEquals(results[1].entry_type, client_pb2.X509_ENTRY)

    def test_issuer_and_root_issuer_populated_from_chain(self):
        self.assertEqual(3, len(CHAIN_DERS))

        report = self.CertificateReportBase()
        report.scan_der_certs([(0, CHAIN_DERS[0], CHAIN_DERS[1:],
                                client_pb2.X509_ENTRY)])
        results = report.report()
        self.assertEqual(len(results), 1)

        issuer_cert = cert_desc.from_cert(cert.Certificate(CHAIN_DERS[1]))
        root_cert = cert_desc.from_cert(cert.Certificate(CHAIN_DERS[2]))

        self.assertEqual(readable_dn(results[0].issuer),
                         'C=US,O=Google Inc,CN=Google Internet Authority')
        self.assertEqual(readable_dn(results[0].root_issuer),
                         'C=US,O=Equifax,OU=Equifax Secure Certificate Authority')

    def test_chain_containing_only_root_handled(self):
        report = self.CertificateReportBase()
        report.scan_der_certs([(0, SELF_SIGNED_ROOT_DER, [], client_pb2.X509_ENTRY)])
        results = report.report()
        self.assertEqual(len(results), 1)
        self.assertEquals(results[0].entry_type, client_pb2.X509_ENTRY)

    def test_issuer_public_key_populated_from_chain(self):
        # Verify the test data is what is expected for this unit test.
        self.assertEqual(3, len(CHAIN_DERS))
        self.assertEqual(
            cert.Certificate(CHAIN_DERS[1]).key_hash(hashfunc="sha256").encode('hex'),
            'b6b95432abae57fe020cb2b74f4f9f9173c8c708afc9e732ace23279047c6d05')

        report = self.CertificateReportBase()
        report.scan_der_certs([(0, CHAIN_DERS[0], CHAIN_DERS[1:],
                                client_pb2.X509_ENTRY)])
        results = report.report()
        self.assertEqual(len(results), 1)

        self.assertEqual(results[0].issuer_pk_sha256_hash.encode('hex'),
            'b6b95432abae57fe020cb2b74f4f9f9173c8c708afc9e732ace23279047c6d05')


if __name__ == '__main__':
    sys.argv = gflags.FLAGS(sys.argv)
    unittest.main()



import calendar
import hashlib
import re
import unicodedata

from ct.crypto import cert
from ct.crypto.asn1 import x509_common
from ct.proto import certificate_pb2


def from_cert(certificate):
    """Pulls out interesting fields from certificate, so format of data will
    be similar in every database implementation."""
    proto = certificate_pb2.X509Description()
    proto.der = certificate.to_der()
    try:
        for sub in [(type_.short_name,
                     to_unicode('.'.join(
                         process_name(value.human_readable(), type_.short_name == 'CN'))))
                    for type_, value in certificate.subject()]:
            proto_sub = proto.subject.add()
            proto_sub.type, proto_sub.value = sub
    except cert.CertificateError:
        pass

    try:
        for iss in [(type_.short_name,
                     to_unicode('.'.join(process_name(value.human_readable(), False))))
                    for type_, value in certificate.issuer()]:
            proto_iss = proto.issuer.add()
            proto_iss.type, proto_iss.value = iss
    except cert.CertificateError:
        pass

    try:
        for alt in certificate.subject_alternative_names():
            proto_alt = proto.subject_alternative_names.add()
            proto_alt.type, proto_alt.value = (alt.component_key(),
                                               to_unicode('.'.join(process_name(
                                      alt.component_value().human_readable()))))
    except cert.CertificateError:
        pass

    try:
        proto.version = str(certificate.version())
    except cert.CertificateError:
        pass

    try:
        proto.serial_number = str(certificate.serial_number().human_readable()
                                  .upper().replace(':', ''))
    except cert.CertificateError:
        pass

    try:
        tbs_alg = certificate.signature()["algorithm"]
        if tbs_alg:
            proto.tbs_signature.algorithm_id = tbs_alg.long_name

        tbs_params = certificate.signature()["parameters"]
        if tbs_params:
            proto.tbs_signature.parameters = tbs_params.value

        cert_alg = certificate.signature_algorithm()["algorithm"]
        if cert_alg:
            proto.cert_signature.algorithm_id = cert_alg.long_name

        cert_params = certificate.signature_algorithm()["parameters"]
        if cert_params:
            proto.cert_signature.parameters = cert_params.value
    except cert.CertificateError:
        pass

    try:
        proto.basic_constraint_ca = bool(certificate.basic_constraint_ca())
    except cert.CertificateError:
        pass

    try:
        proto.validity.not_before, proto.validity.not_after = (
            1000 * int(calendar.timegm(certificate.not_before())),
            1000 * int(calendar.timegm(certificate.not_after())))
    except cert.CertificateError:
        pass

    proto.sha256_hash = hashlib.sha256(proto.der).digest()

    return proto


def to_unicode(value):
    encoded = unicode(value, 'utf-8', 'replace')
    for ch in encoded:
        try:
            _ = unicodedata.name(ch)
        except ValueError:
            # Mangled Unicode code-point. Perhaps this is just
            # plain ISO-8859-1 data incorrectly reported as UTF-8.
            return unicode(value, 'iso-8859-1', 'replace')
    return encoded


def process_name(subject, reverse=True):
    # RFCs for DNS names: RFC 1034 (sect. 3.5), RFC 1123 (sect. 2.1);
    # for common names: RFC 5280.
    # However we probably do not care about full RFC compliance here
    # (e.g. we ignore that a compliant label cannot begin with a hyphen,
    # we accept multi-wildcard names, etc.).
    #
    # For now, make indexing work for the common case:
    # allow letter-digit-hyphen, as well as wildcards (RFC 2818).
    forbidden = re.compile(r"[^a-z\d\-\*]")
    labels = subject.lower().split(".")
    valid_dns_name = len(labels) > 1 and all(
        map(lambda x: len(x) and not forbidden.search(x), labels))

    if valid_dns_name:
        # ["com", "example", "*"], ["com", "example", "mail"],
        # ["localhost"], etc.
        return list(reversed(labels)) if reverse else labels

    else:
        # ["John Smith"], ["Trustworthy Certificate Authority"],
        # ["google.com\x00"], etc.
        # TODO(ekasper): figure out what to do (use stringprep as specified
        # by RFC 5280?) to properly handle non-letter-digit-hyphen names.
        return [subject]



import abc

from ct.client.db import temp_db
from ct.client.db import database
from ct.proto import client_pb2

# This class provides common tests for all CT log TempDB implementations.
# It only inherits from object so that unittest won't attempt to run the test_*
# methods on this class. Derived classes should use multiple inheritance
# from TempDBTest and unittest.TestCase to get test automation.
class TempDBTest(object):
    """All TempDB tests should derive from this class as well as
    unittest.TestCase."""
    __metaclass__ = abc.ABCMeta

    @staticmethod
    def make_entries(start, end):
        entries = []
        for i in range(start, end+1):
            entry = client_pb2.EntryResponse()
            entry.leaf_input = "leaf_input-%d" % i
            entry.extra_data = "extra_data-%d" % i
            entries.append((i, entry))
        return entries

    def verify_entries(self, entries, start, end, prefix=""):
        self.assertEqual(end-start+1, len(entries))
        for i in range(start, end+1):
            self.assertEqual("leaf_input-%d" % i, entries[i].leaf_input)
            self.assertEqual("extra_data-%d" % i, entries[i].extra_data)

    @abc.abstractmethod
    def db(self):
        """Derived classes must override to initialize a database."""

    def test_store_and_scan(self):
        entries = self.make_entries(0, 9)
        self.db().store_entries(entries)
        returned_entries = list(self.db().scan_entries(0, 9))
        self.verify_entries(returned_entries, 0, 9)

    def test_store_twice_doesnt_raise(self):
        entries = self.make_entries(0, 9)
        self.db().store_entries(entries)
        entries = self.make_entries(1, 10)
        self.db().store_entries(entries)
        self.verify_entries(list(self.db().scan_entries(0, 10)), 0, 10)

    def test_scan_out_of_range_fails(self):
        entries = self.make_entries(0, 9)
        self.db().store_entries(entries[:-1])
        returned_entries = self.db().scan_entries(0, 9)
        for _ in range(9):
            returned_entries.next()
        self.assertRaises(database.KeyError, returned_entries.next)
